@@ -1,9 +1,10 @@
 #include <stdio.h>
+#include "chasen.h"
 
-main()
+char *opt[] = {"chasmpl", "-F", "%m / %y / %P- / %T  / %F \n", NULL};
+
+int main()
 {
-    char *opt[] = {"chasmpl", "-F", "%m ", NULL};
-
     if (chasen_getopt_argv(opt, stdout))
       exit(1);
 
@@ -11,6 +12,8 @@ main()
     while (!chasen_fparse(stdin, stdout))
       ;
 #else
-    chasen_sparse("ˤϡ", stdout);
+    chasen_sparse("䥤ǤΩƤޤ", stdout);
 #endif
+
+    return 0;
 }
