@@ -6,6 +6,8 @@
                                           >>> 94/02 changed by T.Nakamura <<<
 ==============================================================================
 */
+#ifndef __CHADIC_H__
+#define __CHADIC_H__
 
 #include <stdio.h>
 #include <ctype.h>
@@ -14,7 +16,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef _WIN32
+#if defined _WIN32 && ! defined __CYGWIN__
 #include <io.h>
 #else
 #include <sys/file.h>
@@ -25,42 +27,51 @@
 #include <fcntl.h>
 
 #include "config.h"
+#include "pat.h"
+#include "sufary.h"
 
 /*
- * macros
+grep * macros
  */
 
 #ifndef FALSE
-#define         FALSE           ((int)(0))
+#define FALSE  ((int)0)
 #endif
 #ifndef TRUE
-#define         TRUE            (!(FALSE))
+#define TRUE   (!FALSE)
 #endif
 
-#ifdef _WIN32
+#if defined _WIN32
 #define PATH_DELIMITER  '\\'
+#define PATHTYPE_MSDOS
 #else
 #define PATH_DELIMITER  '/'
 #endif
 
+/* fseek() */
 #ifndef SEEK_SET
-#define SEEK_SET        0       /* use in "fseek()"         */
-#define SEEK_CUR        1       /* use in "fseek()"         */
-#define SEEK_END        2       /* use in "fseek()"         */
+#define SEEK_SET  0
+#define SEEK_CUR  1
+#define SEEK_END  2
 #endif
 
-#define CHA_FILENAME_MAX 1024
+#define CHAINT_OFFSET   11
+#define CHAINT_SCALE    (256-CHAINT_OFFSET)
 
-#define MIDASI_MAX	129		/* maximum length of MIDASI */
+#define CHA_FILENAME_LEN 1024
 
-#define CLASSIFY_NO	128
+#define MIDASI_LEN	129
 
-#define TYPE_NO		256
-#define FORM_NO		128
+#define HINSI_NUM	128
 
-#define JSTR_BUNTO           "ʸƬ"
-#define JSTR_BUNMATSU        "ʸ"
-#define JSTR_BKUGIRI         ""
+#define TYPE_NUM		256
+#define FORM_NUM		128
+
+#define JSTR_BOS             "ʸƬ"
+#define ESTR_BOS             "BOS"
+#define JSTR_EOS             "ʸ"
+#define ESTR_EOS             "EOS"
+#define JSTR_BKUGIRI         "/"
 #define ESTR_BOS_EOS         "BOS/EOS"
 
 /* cforms.cha */
@@ -70,6 +81,8 @@
 /* *.dic */
 #define JSTR_DEF_POS_COST  "ǥեʻ쥳"
 #define ESTR_DEF_POS_COST  "DEF_POS_COST"
+#define JSTR_MRPH          ""
+#define ESTR_MRPH          "MORPH"
 #define JSTR_POS           "ʻ"
 #define ESTR_POS           "POS"
 #define JSTR_WORD          "Ф"
@@ -82,8 +95,15 @@
 #define ESTR_PRON          "PRON"
 #define JSTR_CTYPE         "ѷ"
 #define ESTR_CTYPE         "CTYPE"
-#define JSTR_INFO          "̣"
+#define JSTR_CFORM         "ѷ"
+#define ESTR_CFORM         "CFORM"
+#define JSTR_INFO1         "ղþ"
+#define JSTR_INFO2         "̣"
 #define ESTR_INFO          "INFO"
+#define JSTR_COMPO         "ʣ"
+#define ESTR_COMPO         "COMP"
+#define JSTR_SEG           ""
+#define ESTR_SEG           "SEG"
 #define JSTR_CONN_ATTR     "Ϣ°"
 
 /* chasenrc */
@@ -92,11 +112,7 @@
 #define JSTR_UNKNOWN_WORD1  "̤θ"
 #define JSTR_UNKNOWN_WORD2  "̤"
 #define ESTR_UNKNOWN_WORD   "UNKNOWN"
-#ifdef VGRAM
 #define JSTR_UNKNOWN_WORD   JSTR_UNKNOWN_WORD1
-#else
-#define JSTR_UNKNOWN_WORD   JSTR_UNKNOWN_WORD2
-#endif
 #define JSTR_UNKNOWN_POS1   "̤θʻ"
 #define JSTR_UNKNOWN_POS2   "̤ʻ"
 #define ESTR_UNKNOWN_POS    "UNKNOWN_POS"
@@ -116,6 +132,8 @@
 #define ESTR_DEF_CONN_COST  "DEF_CONN_COST"
 #define JSTR_COMPO_POS      "Ϣʻ"
 #define ESTR_COMPO_POS      "COMPOSIT_POS"
+#define JSTR_OUTPUT_COMPO   "ʣ"
+#define ESTR_OUTPUT_COMPO   "OUTPUT_COMPOUND"
 #define ESTR_PAT_FILE       "PATDIC" /* changed by Tatuo 960920 */
 #define ESTR_SUF_FILE       "SUFDIC"
 #define JSTR_OUTPUT_FORMAT  "ϥեޥå"
@@ -143,52 +161,31 @@
 #define CHA_CTYPE_FILE    "chasen.ctypes"
 #define CHA_TABLE_FILE    "chasen.table"
 #define CHA_MATRIX_FILE   "chasen.matrix"
-#define JM_CONNECT_FILE   "JUMAN.connect.c"
-#define JM_CONNTMP_FILE   "JUMAN.connect"
-#define JM_GRAMMAR_FILE   "JUMAN.grammar"
-#define JM_CFORM_FILE     "JUMAN.katuyou"
-#define JM_CTYPE_FILE     "JUMAN.kankei"
-#define JM_TABLE_FILE     "JUMANTREE.table"
-#define JM_MATRIX_FILE    "JUMANTREE.matrix"
-#ifdef VGRAM
-#define CONNECT_FILE	VCHA_CONNECT_FILE
-#define CONNTMP_FILE	VCHA_CONNTMP_FILE
-#define GRAMMAR_FILE	VCHA_GRAMMAR_FILE
-#define CFORM_FILE	VCHA_CFORM_FILE
-#define CTYPE_FILE	VCHA_CTYPE_FILE
-#define TABLE_FILE	VCHA_TABLE_FILE
-#define MATRIX_FILE	VCHA_MATRIX_FILE
-#else
-#define CONNECT_FILE	CHA_CONNECT_FILE
-#define CONNTMP_FILE	CHA_CONNTMP_FILE
-#define GRAMMAR_FILE	CHA_GRAMMAR_FILE
-#define CFORM_FILE	CHA_CFORM_FILE
-#define CTYPE_FILE	CHA_CTYPE_FILE
-#define TABLE_FILE	CHA_TABLE_FILE
-#define MATRIX_FILE	CHA_MATRIX_FILE
-#endif
+#define CONNECT_FILE	  VCHA_CONNECT_FILE
+#define CONNTMP_FILE	  VCHA_CONNTMP_FILE
+#define GRAMMAR_FILE	  VCHA_GRAMMAR_FILE
+#define CFORM_FILE	  VCHA_CFORM_FILE
+#define CTYPE_FILE	  VCHA_CTYPE_FILE
+#define TABLE_FILE	  VCHA_TABLE_FILE
+#define MATRIX_FILE	  VCHA_MATRIX_FILE
 
 #define CONS		0
 #define ATOM		1
-#define NIL		((cell_t *)(NULL))
+#define NIL		((chasen_cell_t *)(NULL))
 
-#define s_tag(cell)	(((cell_t *)(cell))->tag)
+#define s_tag(cell)	(((chasen_cell_t *)(cell))->tag)
 #define consp(x)	(!nullp(x) && (s_tag(x) == CONS))
 #define atomp(x)	(!nullp(x) && (s_tag(x) == ATOM))
 #define nullp(cell)	((cell) == NIL)
-#define car_val(cell)	(((cell_t *)(cell))->value.cons.car)
-#define cdr_val(cell)	(((cell_t *)(cell))->value.cons.cdr)
-#define s_atom_val(cell) (((cell_t *)(cell))->value.atom)
+#define car_val(cell)	(((chasen_cell_t *)(cell))->value.cha_cons.cha_car)
+#define cdr_val(cell)	(((chasen_cell_t *)(cell))->value.cha_cons.cha_cdr)
+#define s_atom_val(cell) (((chasen_cell_t *)(cell))->value.atom)
 
 /* added by T.Utsuro for weight of rensetu matrix */
 #define DEFAULT_C_WEIGHT  10
 
 /* added by S.Kurohashi for mrph weight default values */
-#ifdef VGRAM
 #define MRPH_DEFAULT_WEIGHT	1
-#else
-#define MRPH_DEFAULT_WEIGHT	10
-#endif
 
 #define strmatch2(s,s1,s2)      (!strcmp(s,s1)||!strcmp(s,s2))
 #define strmatch3(s,s1,s2,s3)   (!strcmp(s,s1)||!strcmp(s,s2)||!strcmp(s,s3))
@@ -203,47 +200,41 @@ typedef struct _connect_rule_t {
  * structures
  */
 
-/* <car>  <cdr> ؤΥݥ󥿤ɽ줿 */
+/* <cha_car>  <cha_cdr> ؤΥݥ󥿤ɽ줿 */
 typedef struct _bin_t {
-    void *car;			/* address of <car> */
-    void *cdr;			/* address of <cdr> */
+    void *cha_car;			/* address of <cha_car> */
+    void *cha_cdr;			/* address of <cha_cdr> */
 } bin_t;
 
 /* <BIN> ޤ ʸ ɽ봰ʹ¤ */
 typedef struct _cell {
-    int tag;			/* tag of <cell> 0:cons 1:atom */
+    int tag;			/* tag of <cell> 0:cha_cons 1:atom */
     union {
-	bin_t	cons;
+	bin_t	cha_cons;
 	char	*atom;
     } value;
-} cell_t;
+} chasen_cell_t;
 
 /* changed by T.Nakamura and S.Kurohashi 
-	¤ mrph_t ٤Ƥξ 
-	¤ MORPHEME Ϥʤʤä */
+   ¤ mrph_t ٤Ƥξ 
+   ¤ MORPHEME Ϥʤʤä */
 typedef struct _mrph {
-    char	midasi[MIDASI_MAX];
-    char	yomi[MIDASI_MAX];
-    char	*info;
-#ifdef VGRAM
-    char	*base;
-    char	pron[MIDASI_MAX];
+    char midasi[MIDASI_LEN];
+    char yomi[MIDASI_LEN];
+    char *info;
+    char *base;
+    char pron[MIDASI_LEN];
     unsigned short hinsi;
-#else
-    unsigned char hinsi;
-    unsigned char bunrui;
-#endif
     unsigned char ktype;
     unsigned char kform;
 
-    short	con_tbl;
-    short	length;
+    short con_tbl;
+    short length;
     unsigned short weight;
 
-    char	is_undef;
+    char is_undef;
 } mrph_t;
 
-#ifdef VGRAM
 typedef struct _hinsi_t {
     short *path;
     short *daughter;
@@ -254,15 +245,6 @@ typedef struct _hinsi_t {
     char  kt;
     unsigned char cost;
 } hinsi_t;
-#else
-/* ʻʬࡦʬ */
-typedef struct _class {
-    char *id;
-    unsigned char comp; /* 1:ʬबϢʻ 2:ʬबϢʻ */
-    char kt;
-    unsigned char cost; /*ʻ쥳 by k.n*/
-} class_t;
-#endif
 
 /* ѷ */
 typedef struct _ktype {
@@ -274,10 +256,9 @@ typedef struct _ktype {
 typedef struct _kform {
     char  *name;
     char  *gobi;
+    int   gobi_len;
     char  *ygobi;
-#ifdef VGRAM
     char  *pgobi;
-#endif
 } kform_t;
 
 /* Ϣɽ */
@@ -286,31 +267,21 @@ typedef struct _rensetu_pair {
     short  i_pos;
     short  j_pos;
 
-#ifdef VGRAM
     unsigned short hinsi;
-#else
-    unsigned char hinsi;
-    unsigned char bunrui;
-#endif
     unsigned char type;
     unsigned char form;
     char   *goi;
 } rensetu_pair_t;
 
-
 /*
  * global variables
  */
 
-#ifdef VGRAM
 #define HINSI_MAX     4096
-extern hinsi_t Hinsi[HINSI_MAX];
-#else
-extern class_t Class[CLASSIFY_NO][CLASSIFY_NO];
-#endif
-extern ktype_t Type[TYPE_NO];
-extern kform_t Form[TYPE_NO][FORM_NO];
-extern int LineNo, LineNoForError;
+extern hinsi_t Cha_hinsi[HINSI_MAX];
+extern ktype_t Cha_type[TYPE_NUM];
+extern kform_t Cha_form[TYPE_NUM][FORM_NUM];
+extern int Cha_lineno, Cha_lineno_error;
 
 /* getopt.c */
 extern int Cha_optind;
@@ -320,88 +291,91 @@ extern int Cha_server_mode;
 extern int Cha_errno;
 extern FILE *Cha_stderr;
 
+/* dictionaries(dic.c) */
+extern SUFARY *Suf_dicfile[];
+extern pat_t *Pat_dicfile[];
+extern int Suf_ndicfile;
+extern int Pat_ndicfile;
+
 /*
  * functions
  */
 
 /* iotool.c */
-extern char *convert_escape();
-extern void set_progpath();
-extern void set_chasenrc_path();
-extern char *get_chasenrc_path();
-extern char *get_grammar_dir();
-extern void set_filepath();
-extern FILE *cha_fopen();
-extern FILE *cha_fopen2();
-extern FILE *cha_fopen_grammar();
-extern FILE *cha_fopen_grammar2();
-extern int  cha_feof();
-extern void *cha_malloc();
-extern char *cha_strdup();
-
-extern void cha_fexit();
-extern void cha_exit();
-extern void cha_exit_file();
-extern void cha_perror();
-extern void cha_exit_perror();
-extern FILE *fopen_chasenrc();
-extern void read_grammar_dir();
+char *cha_convert_escape(char*, int);
+void cha_set_progpath(char*);
+void cha_set_rcpath(char*);
+char *cha_get_rcpath(void);
+char *cha_get_grammar_dir(void);
+void cha_set_filepath(char*);
+FILE *cha_fopen(char*, char*, int);
+FILE *cha_fopen2(char*, char*, char*, int);
+FILE *cha_fopen_grammar(char*, char*, int, int, char**);
+FILE *cha_fopen_grammar2(char*, char*, char*, int, int, char**);
+void *cha_malloc(size_t);
+#define cha_free(ptr) free(ptr)
+char *cha_strdup(char*);
 
+void cha_exit(int, char*, ...);
+void cha_exit_file(int, char*, ...);
+void cha_perror(char*);
+void cha_exit_perror(char*);
+FILE *cha_fopen_rcfile(void);
+void cha_read_grammar_dir(void);
 
 /* lisp.c */
-extern void set_cha_getc_alone();
-extern void set_cha_getc_server();
-extern void set_skip_char();
-extern int s_feof();
-extern cell_t *tmp_atom();
-extern cell_t *cons();
-extern cell_t *car();
-extern cell_t *cdr();
-extern char *s_atom();
-extern int equal();
-extern int s_length();
-extern cell_t *s_read();
-extern cell_t *assoc();
-extern char *s_tostr();
-extern cell_t *s_print();
+void cha_set_getc_alone(void);
+void cha_set_getc_server(void);
+void cha_set_skip_char(int);
+int cha_s_feof(FILE*);
+void cha_s_free(chasen_cell_t*);
+chasen_cell_t *cha_tmp_atom(char*);
+chasen_cell_t *cha_cons(void*, void*);
+chasen_cell_t *cha_car(chasen_cell_t*);
+chasen_cell_t *cha_cdr(chasen_cell_t*);
+char *cha_s_atom(chasen_cell_t*);
+int cha_equal(void*, void*);
+int cha_s_length(chasen_cell_t*);
+chasen_cell_t *cha_s_read(FILE*);
+chasen_cell_t *cha_assoc(chasen_cell_t*, chasen_cell_t*);
+char *cha_s_tostr(chasen_cell_t*);
+chasen_cell_t *cha_s_print(FILE*, chasen_cell_t*);
 
 /* grammar.c */
-extern int match_nhinsi();
-extern void read_grammar();
+void cha_read_class(FILE*);
+int cha_match_nhinsi(chasen_cell_t*, int);
+void cha_read_grammar(FILE*, int, int);
 
 /* katuyou.c */
-extern void read_katuyou();
+void cha_read_katuyou(FILE*, int);
 
 /* connect.c */
-extern void read_table();
-extern int check_table(); /* 970301 tatuo: void -> int for  */
-extern int check_table_for_undef();
-extern void read_matrix();
-extern int check_automaton();
+void cha_read_table(FILE*, int);
+int cha_check_table(mrph_t*); /* 970301 tatuo: void -> int for  */
+int cha_check_table_for_undef(int);
+void cha_read_matrix(FILE*);
+int cha_check_automaton(int, int, int, int*);
 /* for EDR dic */
-extern void check_edrtable();
-extern void check_edrtable_str();
+void cha_check_edrtable(mrph_t *, chasen_cell_t*);
+void cha_check_edrtable_str(mrph_t*, char*); /* Unused.  */
 
 /* getid.c */
-#ifdef VGRAM
-extern int get_nhinsi_str_id();
-extern int get_nhinsi_id();
-#else
-extern int get_hinsi_id();
-extern int get_bunrui_id();
-extern void get_hinsi_bunrui_id();
-#endif
-extern int get_type_id();
-extern int get_form_id();
+int cha_get_nhinsi_str_id(char**);
+int cha_get_nhinsi_id(chasen_cell_t*);
+int cha_get_type_id(char*);
+int cha_get_form_id(char*, int);
 
 /* zentohan.c */
-extern char *zentohan();
-extern char *hantozen();
-extern unsigned char *euc2sjis();
-extern unsigned char *sjis2euc();
-extern unsigned char *hankana2zenkana();
+unsigned char *euc2sjis(char*);
+unsigned char *sjis2euc(unsigned char*);
+unsigned char *hankana2zenkana(unsigned char*);
 
 /* getopt.c */
-extern int cha_getopt();
-extern int cha_getopt_chasen();
+int cha_getopt(char**, char*, FILE*);
+int cha_getopt_chasen(char**, FILE*);
 
+/* mmap.c */
+off_t cha_mmap_file(char*, void**);
+void cha_munmap_file(void*, off_t);
+
+#endif /* __CHADIC_H__ */
