@@ -6,6 +6,7 @@
 
 #include "chalib.h"
 #include "pat.h"
+#include "sufary.h"
 
 #ifdef KOCHA
 #define CHA_NAME       "KoCha"
@@ -13,15 +14,14 @@
 #define CHA_NAME       "ChaSen"
 #endif
 
-int Cost_width;
-int Lang_j = 0, Lang_e = 0;
+int Cha_cost_width = -1;
+int Cha_lang_j = 0, Cha_lang_e = 0;
 
 static int cost_width0;
-/* ե̾ */
-static char patdic_filename[MAX_DIC_NUMBER][CHA_FILENAME_MAX];
-static char sufdic_filename[MAX_DIC_NUMBER][CHA_FILENAME_MAX];
-char *sufdic_file[MAX_DIC_NUMBER];
-int num_sufdic_file = 0;
+
+static char patdic_filename[PAT_DIC_NUM][CHA_FILENAME_LEN];
+static char sufdic_filename[PAT_DIC_NUM][CHA_FILENAME_LEN];
+
 static int obj_dic_no = 0; /* ưŪ(ɲ)оݤȤʤ뼭ֹ */
 
 static int opt_show = 'b', opt_form = 'f', opt_ja, opt_cmd, opt_nobk;
@@ -36,19 +36,15 @@ void cha_version(fp)
     if (!fp)
       return;
 
-    fprintf(fp, "%s version %s (c) 1996-1999 Nara Institute of Science and Technology\n",
+    fprintf(fp, "%s version %s (c) 1996-2000 Nara Institute of Science and Technology\n",
 	    CHA_NAME, VERSION);
-#ifndef VGRAM
-    fprintf(fp, "Grammar files are compatible with JUMAN 2.0.\n");
-#else
     fprintf(fp, "Grammar files are in ChaSen's new v-gram format.\n");
-#endif
 }
 
 /*
- * set_opt_form()
+ * cha_set_opt_form()
  */
-void set_opt_form(format)
+void cha_set_opt_form(format)
     char *format;
 {
     char *f;
@@ -69,7 +65,6 @@ void set_opt_form(format)
 	if (opt_form == 'd' || opt_form == 'v')
 	  opt_show = 'm';
         switch (opt_form) {
-#ifdef VGRAM
 	  case 'd':
 	    opt_form_string = "morph(%pi,%ps,%pe,%pc,'%m','%U(%y)','%M',%U(%P'),NIL,%T0,%F0,'%I0',%c,[%ppc,],[%ppi,])";
 	    break;
@@ -85,30 +80,13 @@ void set_opt_form(format)
 	  case 'c':
 	    opt_form_string = "%m\t%y\t%M\t%h %t %f\n";
 	    break;
-#else
-	  case 'd':
-	    opt_form_string = "morph(%pi,%ps,%pe,%pc,'%m','%U(%y)','%M',%H,%BB,%T0,%F0,'%I0',%c,[%ppc,],[%ppi,])";
-	    break;
-	  case 'v':
-	    opt_form_string = "%3pi %3ps %3pe %5pc %m %U(%y) %M %H %BB %T0 %F0 %I0 %c %ppi, %ppc,\n";
-	    break;
-	  case 'f':
-	    opt_form_string = "%-11m %-11y %-11M %-14U(%BB) %-14T  %F \n";
-	    break;
-	  case 'e':
-	    opt_form_string = "%m %U(%y) %M %H %h %B* %b %T* %t %F* %f\n";
-	    break;
-	  case 'c':
-	    opt_form_string = "%m %y %M %h %b %t %f\n";
-	    break;
-#endif
 	}
 	return;
     }
 
     /* format string */
-    /*    opt_form_string = convert_escape(cha_strdup(format), 1);*/
     opt_form_string = format;
+    /*    opt_form_string = cha_convert_escape(cha_strdup(format), 1); */
 
     f = opt_form_string + strlen(opt_form_string);
     if (f[-1] == '\n')
@@ -118,33 +96,32 @@ void set_opt_form(format)
 }
 
 /*
- * set_language()
+ * cha_set_language()
  */
-void set_language(langstr)
+void cha_set_language(langstr)
     char *langstr;
 {
     char *s;
 
-    Lang_j = Lang_e = 0;
+    Cha_lang_j = Cha_lang_e = 0;
     for (s = langstr; *s; s++) {
 	if (*s == 'j')
-	  Lang_j = 1;
+	  Cha_lang_j = 1;
 	else if (*s == 'e')
-	  Lang_e = 1;
+	  Cha_lang_e = 1;
     }
 }
 
 /*
- * set_cost_width()
+ * cha_set_cost_width()
  */
-void set_cost_width(cw)
+void cha_set_cost_width(cw)
     int cw;
 {
     cost_width0 = cw * MRPH_DEFAULT_WEIGHT;
 
-    /* ŬʳɽȤ Cost_width  */
-    if (opt_show != 'b')
-      Cost_width = cost_width0;
+    /* ŬʳɽȤ Cha_cost_width  */
+    Cha_cost_width = opt_show == 'b' ? -1 : cost_width0;
 }
 
 /*
@@ -166,15 +143,15 @@ int chasen_getopt_argv(argv, fp)
 	switch (c) {
 	  case 'r':
 	    /* chasenrc file */
-	    set_chasenrc_path(Cha_optarg);
+	    cha_set_rcpath(Cha_optarg);
 	    break;
 	  case '?': return 1;
 	}
     }
 
     /* initialize if not done */
-    if (!Undef_info_num)
-      chasen_init();
+    if (!Cha_undef_info_num)
+      cha_init();
 
     /* read options */
     Cha_optind = 0;
@@ -189,22 +166,25 @@ int chasen_getopt_argv(argv, fp)
 	  case 'e':
 	  case 'c':
 	    opt_form = c;
-	    set_opt_form(NULL);
+	    cha_set_opt_form(NULL);
 	    break;
 	  case 'F':
-	    set_opt_form(convert_escape(cha_strdup(Cha_optarg), 0));
+	    cha_set_opt_form(cha_convert_escape(cha_strdup(Cha_optarg), 0));
 	    break;
 	  case 'L':
-	    set_language(Cha_optarg);
+	    cha_set_language(Cha_optarg);
 	    break;
 	  case 'w': /* λ */
-	    set_cost_width(atoi(Cha_optarg));
+	    cha_set_cost_width(atoi(Cha_optarg));
+	    break;
+	  case 'O':
+	    Cha_output_compo = *Cha_optarg == 'c';
 	    break;
 	  case 'l':
-	    set_cha_output(stdout);
+	    cha_set_output(stdout);
 	    switch (*Cha_optarg) {
 	      case 'p':
-		/* display the list of Hinsi table */
+		/* display the list of Cha_hinsi table */
 		cha_print_hinsi_table();
 		exit(0);
 		break;
@@ -229,9 +209,8 @@ int chasen_getopt_argv(argv, fp)
 	}
     }
 
-    /* ŬʳɽȤ Cost_width  */
-    if (opt_show != 'b')
-      Cost_width = cost_width0;
+    /* ŬʳɽȤ Cha_cost_width  */
+    Cha_cost_width = opt_show == 'b' ? -1 : cost_width0;
 
     return 0;
 }
@@ -289,7 +268,7 @@ static int chasen_command(comm)
     char *comm;
 {
     char *arg;
-    int i, j;
+    int i;
     char *rslt[256]; /* ѿ for ñå(exact match) */
     FILE *of; /* intե˽񤭹(ɲ)Τ */
     long new_word_index;
@@ -303,29 +282,29 @@ static int chasen_command(comm)
 	cha_version(stdout);
 	break;
       case 'F':
-	set_opt_form(convert_escape(cha_strdup(arg), 0));
+	cha_set_opt_form(cha_convert_escape(cha_strdup(arg), 0));
 	break;
       case 'L':
-	set_language(Cha_optarg);
+	cha_set_language(Cha_optarg);
 	break;
       case 'w':
 	/* cost width */
-	set_cost_width(atoi(arg));
+	cha_set_cost_width(atoi(arg));
 	break;
       case 'i':
 	/* information */
 	cha_version(stdout);
-	printf("\n()        %d\n",Cost_width);
-	printf("ϢܥȽŤ()  %d\n",Con_cost_weight);
-	printf("ǥȽŤ      %d\n",Mrph_cost_weight);
-	printf("ϥեޥå      %s",
-	       opt_form_string ? opt_form_string : "(ʤ)");
-	printf("\nchasenrcե\n\t%s\n",get_chasenrc_path());
-	printf("ʸˡե\n\t%s\n",get_grammar_dir());
-	printf("ե\n");
+	printf("\ncost width:           %d\n",Cha_cost_width);
+	printf("weight of conn. cost: %d\n",Cha_con_cost_weight);
+	printf("weight of morph cost: %d\n",Cha_mrph_cost_weight);
+	printf("output format:        \"%s\"\n",
+	       opt_form_string ? opt_form_string : "(none)");
+	printf("chasenrc file:        %s\n", cha_get_rcpath());
+	printf("grammar file:         %s\n",cha_get_grammar_dir());
+	printf("dic file:\n");
 	for(i = 0; patdic_filename[i][0]; i++)
 	  printf("\t%s\n",patdic_filename[i]);
-	printf("ưŪоݤȤʤ뼭\n\t%s\n",
+	printf("dic file for processing:\n\t%s\n",
 	       patdic_filename[obj_dic_no]);
 	break;
       case 'f':
@@ -343,16 +322,6 @@ static int chasen_command(comm)
       case 'a':
 	/* ѥڤؤñɲá*/
 	if(strlen(arg) < 4){printf("invalid format\n");break;}
-	/* ե礭 = Υǥå */
-#ifdef NO_MMAP
-	{
-	    struct stat st;
-	    fstat(fileno(dic_file[obj_dic_no]), &st);
-	    new_word_index = st.st_size;
-	}
-#else
-	new_word_index = dicinfo[obj_dic_no].size;
-#endif
 	/* ɲäñintեɲ */
 	sprintf(tmpstr,"%s.int",patdic_filename[obj_dic_no]);
 	of = cha_fopen(tmpstr,"a",1);
@@ -360,42 +329,32 @@ static int chasen_command(comm)
 	fputc(0, of);
 	printf("add [%s] at %ld\n", arg, new_word_index);
 	fclose(of);
-	/* ɤ߹ѤintեĤƤޤ */
-	fclose(dic_file[obj_dic_no]);
-	dic_file[obj_dic_no] = cha_fopen(tmpstr, "r", 1);
 	/* ޥåפľƤ餦ν */
-	dicinfo[obj_dic_no].used = 0;
-	(void)pat_insert(dic_file[obj_dic_no],
-			 arg, new_word_index, /*113, */
-			 &tree_top[obj_dic_no], "\t");
+	pat_text_reopen(Pat_dicfile[obj_dic_no], tmpstr);
+	pat_insert(Pat_dicfile[obj_dic_no],
+		   arg, new_word_index, "\t");
 	break;
       case 's':
 	/* ڤΥ */
 	sprintf(tmpstr,"%s.pat",patdic_filename[obj_dic_no]);
-	(void)com_s(tmpstr,&tree_top[obj_dic_no]);
+	pat_save(Pat_dicfile[obj_dic_no], tmpstr);
 	break;
       case 'e':
 	/* θ (exact match) */
 	for(i = 0; patdic_filename[i][0]; i++){
 	    mrph2_t mrph;
 	    printf("DIC No. %d   \"%s\"\n",i,patdic_filename[i]);
-	    (void)pat_search_exact(dic_file[i],arg, &tree_top[i],rslt);
+	    pat_search_exact(Pat_dicfile[i], arg, rslt);
 	    if(!rslt[0]) printf("Not Found.\n");
 	    else {
 		char **pbuf;
 		for (pbuf = rslt; *pbuf; pbuf++) {
-		    take_data(&mrph, *pbuf, arg);
-		    if (
-#ifdef VGRAM
-			Hinsi[mrph.hinsi].kt &&
-#else
-			Class[mrph.hinsi][mrph.bunrui].kt &&
-#endif
-			mrph.kform) {
+		    cha_get_mrph_data(&mrph, *pbuf, arg);
+		    if (Cha_hinsi[mrph.hinsi].kt && mrph.kform) {
 			mrph.base_length = 0;
 			mrph.yomi = "";
 		    }
-		    printf_mrph(0, &mrph, opt_form_string);
+		    cha_printf_mrph(0, &mrph, opt_form_string);
 		}
 	    }
 	}
@@ -429,23 +388,20 @@ static int chasen_sparse_main(input, output)
     char *crlf;
 
     /* initialize if not done */
-    if (!Undef_info_num)
-      chasen_init();
+    if (!Cha_undef_info_num)
+      cha_init();
     if (!opt_form_string)
-      set_opt_form(NULL);
+      cha_set_opt_form(NULL);
 
 #if 0
     /* βԥɤ */
     chomp(input);
 #endif
 
-    set_cha_output(output);
+    cha_set_output(output);
 
     if (input[0] == '\0') {
-#ifdef VGRAM
-	if (!opt_ja)
-	  cha_print_bos_eos(opt_form);
-#endif
+	cha_print_bos_eos(opt_form);
 	return 0;
     }
 
@@ -458,7 +414,7 @@ static int chasen_sparse_main(input, output)
 
     /* parse a sentence and print */
     while (*input) {
-	int c, len;
+	int c = 0, len;
 	if ((crlf = strpbrk(input, "\r\n")) == NULL)
 	  len = strlen(input);
 	else {
@@ -469,13 +425,11 @@ static int chasen_sparse_main(input, output)
 #ifdef SJIS
 	sjis2euc(input);
 #endif
-	if (len > 0 && !chasen_sent(input, len, opt_nobk)) {
+	if (len > 0 && !cha_parse_sentence(input, len, opt_nobk)) {
 	    cha_print_path(opt_show, opt_form, opt_form_string);
-	}
-#ifdef VGRAM
-	else if (!opt_ja)
+	} else if (!opt_ja)
 	  cha_print_bos_eos(opt_form);
-#endif
+
 	if (crlf == NULL)
 	  break;
 	if (c == '\r' && crlf[1] == '\n')
@@ -547,7 +501,7 @@ char *chasen_fparse_tostr(fp_in)
     if (chasen_sparse_main(line, NULL))
       return NULL;
 
-    return get_cha_output();
+    return cha_get_output();
 }
 
 /*
@@ -566,7 +520,7 @@ char *chasen_sparse_tostr(str_in)
 
     free(euc_str);
 
-    return get_cha_output();
+    return cha_get_output();
 }
 
 char *cha_fgets(s, n, fp)
@@ -575,81 +529,75 @@ char *cha_fgets(s, n, fp)
     FILE *fp;
 {
     if (opt_ja)
-      return jfgets(s, n, fp);
+      return cha_jfgets(s, n, fp);
     else
-      return fget_line(s, n, fp);
+      return cha_fget_line(s, n, fp);
 }
 
 static void set_dic_filename(filename, s)
     char *filename, *s;
 {
-#ifdef _WIN32
-    if (*s == '\\' || *s && s[1] == ':')
+#ifdef PATHTYPE_MSDOS
+    if (*s == PATH_DELIMITER || *s && s[1] == ':')
       strcpy(filename, s);
 #else
-    if (*s == '/')
+    if (*s == PATH_DELIMITER)
       strcpy(filename, s);
 #endif
     else
-      sprintf(filename, "%s%s", get_grammar_dir(), s);
+      sprintf(filename, "%s%s", cha_get_grammar_dir(), s);
 }
 
 /*
- * read_patdic - read patricia dictionaries
+ * cha_read_patdic - read patricia dictionaries
  */
-void read_patdic(cell)
-    cell_t *cell;
+void cha_read_patdic(cell)
+    chasen_cell_t *cell;
 {
     int  num;
-    char filename[CHA_FILENAME_MAX];
+    char patname[CHA_FILENAME_LEN];
+    char textname[CHA_FILENAME_LEN];
 
     /* return if already read */
     if (patdic_filename[0][0])
       return;
 
-    for (num = 0; !nullp(cell); num++, cell = cdr(cell)) {
-	if (num >= MAX_DIC_NUMBER)
+    for (num = 0; !nullp(cell); num++, cell = cha_cdr(cell)) {
+	if (num >= PAT_DIC_NUM)
 	  cha_exit_file(1, "too many patricia dictionary files");
-	set_dic_filename(&patdic_filename[num], s_atom(car(cell)));
+	set_dic_filename(&patdic_filename[num], cha_s_atom(cha_car(cell)));
 
 	/* Υץ */
-	sprintf(filename, "%s.int", patdic_filename[num]);
-	/* .chasenrc ɤ߹ʤΤ cha_fopen ϻȤʤ */
-	if ((dic_file[num] = fopen(filename, "r")) == NULL)
-	  cha_exit_perror(filename);
-
-	sprintf(filename, "%s.pat", patdic_filename[num]);
-	(void)pat_init_tree_top(&tree_top[num]);
-	(void)com_l(filename, &tree_top[num]);
+	sprintf(textname, "%s.int", patdic_filename[num]);
+	sprintf(patname, "%s.pat", patdic_filename[num]);
+	Pat_dicfile[num] = pat_open(textname, patname);
     }
-    number_of_tree = num;
+    Pat_ndicfile = num;
 }
 
 /*
- * read_sufdic - read SUFARY dictionaries
+ * cha_read_sufdic - read SUFARY dictionaries
  */
-void read_sufdic(cell)
-    cell_t *cell;
+void cha_read_sufdic(cell)
+    chasen_cell_t *cell;
 {
     int  num;
-    char *s;
-    char filename[CHA_FILENAME_MAX];
-    char ary_filename[CHA_FILENAME_MAX];
+    char filename[CHA_FILENAME_LEN];
+    char ary_filename[CHA_FILENAME_LEN];
 
     /* return if already read */
     if (sufdic_filename[0][0])
       return;
 
-    for (num = 0; !nullp(cell); num++, cell = cdr(cell)) {
-	if (num >= MAX_DIC_NUMBER)
+    for (num = 0; !nullp(cell); num++, cell = cha_cdr(cell)) {
+	if (num >= PAT_DIC_NUM)
 	  cha_exit_file(1, "too many SUFARY dictionary files");
-	set_dic_filename(&sufdic_filename[num], s_atom(car(cell)));
+	set_dic_filename(&sufdic_filename[num], cha_s_atom(cha_car(cell)));
 
 	/* Υץ */
 	sprintf(filename, "%s.int", sufdic_filename[num]);
 	sprintf(ary_filename, "%s.ary", sufdic_filename[num]);
-	sufdic_file[num] = (char *)sa_openfiles(filename, ary_filename);
+	Suf_dicfile[num] = sa_openfiles(filename, ary_filename);
     }
-    num_sufdic_file = num;
+    Suf_ndicfile = num;
 }
-
