@@ -6,13 +6,14 @@
 		Oct. 1996       A.Kitauchi <akira-k@is.aist-nara.ac.jp>
 ==============================================================================
 */
-
+#include <stdarg.h>
+#include <stdio.h>
 #include "chadic.h"
 
 #ifdef KOCHA
 #define RCFILE "/.kocharc"
 #else
-#ifdef _WIN32
+#ifdef PATHTYPE_MSDOS
 #define RCFILE "\\chasenrc"
 #define RC2FILE "\\chasen2rc"
 #else
@@ -22,19 +23,19 @@
 #endif
 #define OLDRCFILE "/.jumanrc"
 
-int LineNo, LineNoForError;
+int Cha_lineno, Cha_lineno_error;
 int Cha_errno = 0;
 
 static FILE *cha_stderr = NULL;
-static char progpath[CHA_FILENAME_MAX] = "chasen";
-static char filepath[CHA_FILENAME_MAX];
-static char grammar_dir[CHA_FILENAME_MAX];
-static char chasenrc_path[CHA_FILENAME_MAX];
+static char progpath[CHA_FILENAME_LEN] = "chasen";
+static char filepath[CHA_FILENAME_LEN];
+static char grammar_dir[CHA_FILENAME_LEN];
+static char chasenrc_path[CHA_FILENAME_LEN];
 
 /***********************************************************************
- * convert_escape - convert escape characters
+ * cha_convert_escape - convert escape characters
  ***********************************************************************/
-char *convert_escape(str, ctrl_only)
+char *cha_convert_escape(str, ctrl_only)
     char *str;
     int ctrl_only;
 {
@@ -63,22 +64,22 @@ char *convert_escape(str, ctrl_only)
 }
 
 /***********************************************************************
- * set_progpath - set program pathname
+ * cha_set_progpath - set program pathname
  *
  *	progpath is used in cha_exit() and cha_exit_file()
  ***********************************************************************/
-#ifdef _WIN32
+#if defined _WIN32 && ! defined __CYGWIN__
 static void which(filename, path)
     char *filename;
     char *path;
 {
     char *ps, *pe;
     int fl;
-    
+
     strcpy(path, ".\\");
     strcat(path, filename);
     if (fopen(path, "r") != NULL) return;
-    
+
     ps = getenv("PATH");
 
     for (pe = ps, fl = 0;!fl ; pe++) {
@@ -99,11 +100,11 @@ static void which(filename, path)
 }
 #endif /* _WIN32 */
 
-void set_progpath(path)
+void cha_set_progpath(path)
     char *path;
 {
-#ifdef _WIN32
-    if (strchr(path, '\\') != NULL)
+#if defined _WIN32 && ! defined __CYGWIN__
+    if (strchr(path, PATH_DELIMITER) != NULL)
       strcpy(progpath, path);
     else
       which("chasen.exe", progpath);
@@ -113,41 +114,41 @@ void set_progpath(path)
 }
 
 /***********************************************************************
- * set_chasenrc_path - set chasenrc file path
+ * cha_set_rcpath - set chasenrc file path
  *
  *	this function is called when -r option is used.
  ***********************************************************************/
-void set_chasenrc_path(filename)
+void cha_set_rcpath(filename)
     char *filename;
 {
     strcpy(chasenrc_path, filename);
 }    
 
 /***********************************************************************
- * get_chasenrc_path
+ * cha_get_rcpath
  *
  *	called only from chasen.c
  ***********************************************************************/
-char *get_chasenrc_path()
+char *cha_get_rcpath()
 {
     return chasenrc_path;
 }
 
 /***********************************************************************
- * get_grammar_dir
+ * cha_get_grammar_dir
  *
  *	called only from chasen.c
  ***********************************************************************/
-char *get_grammar_dir()
+char *cha_get_grammar_dir()
 {
     return grammar_dir;
 }
 
-void set_filepath(filename)
+void cha_set_filepath(filename)
     char *filename;
 {
     strcpy(filepath, filename);
-    LineNo = LineNoForError = 0;
+    Cha_lineno = Cha_lineno_error = 0;
 }
 
 /***********************************************************************
@@ -170,7 +171,7 @@ FILE *cha_fopen(filename, mode, ret)
 	if (*mode == 'r') {
 	    if (filename != filepath)
 	      strcpy(filepath, filename);
-	    LineNo = LineNoForError = 0;
+	    Cha_lineno = Cha_lineno_error = 0;
 	}
     } else if (ret >= 0)
       cha_exit_perror(filename);
@@ -214,7 +215,6 @@ FILE *cha_fopen_grammar(filename, mode, ret, dir, filepathp)
     int ret, dir;
 {
     FILE *fp;
-    char *s;
 
     *filepathp = filename;
     switch (dir) {
@@ -230,7 +230,7 @@ FILE *cha_fopen_grammar(filename, mode, ret, dir, filepathp)
 	/* ʸˡǥ쥯ȥ꤫ɤ߹ */
 	/* ʸˡǥ쥯ȥ꤬ꤵƤʤ .chasenrc ɤ߹ */
 	if (grammar_dir[0] == '\0')
-	  read_grammar_dir();
+	  cha_read_grammar_dir();
 	sprintf(filepath, "%s%s", grammar_dir, filename);
 	*filepathp = filepath;
 	return cha_fopen(filepath, mode, ret);
@@ -266,18 +266,14 @@ FILE *cha_fopen_grammar2(filename1, filename2, mode, ret, dir, filepathp)
 }
 
 /*
-------------------------------------------------------------------------------
-        PROCEDURE:
-        <cha_malloc>: do "malloc" (library function) and cha_exit processing
-------------------------------------------------------------------------------
-*/
-
+ * cha_malloc()
+ */
 void *cha_malloc(n)
-    int n;
+    size_t n;
 {
     void *p;
 
-    if ((p = (void *)malloc(n)) == NULL)
+    if ((p = malloc(n)) == NULL)
       cha_exit_perror("malloc");
 
     return p;
@@ -313,16 +309,16 @@ char *cha_strdup(str)
 /*
  * cha_exit() - print error messages on stderr and exit
  */
-void set_cha_stderr(fp)
+void cha_set_stderr(fp)
     FILE *fp;
 {
     cha_stderr = fp;
 }
 
-void cha_exit(status, format, a, b, c, d, e, f, g, h)
-    int status;
-    char *format, *a, *b, *c, *d, *e, *f, *g, *h;
+void cha_exit(int status, char *format, ...)
 {
+    va_list ap;
+
     if (Cha_errno)
       return;
 
@@ -333,8 +329,9 @@ void cha_exit(status, format, a, b, c, d, e, f, g, h)
 
     if (progpath)
       fprintf(cha_stderr, "%s: ", progpath);
-    fprintf(cha_stderr, format, a, b, c, d, e, f, g, h);
-
+    va_start(ap, format);
+    vfprintf(cha_stderr, format, ap);
+    va_end(ap);
     if (status >= 0) {
 	fputc('\n', cha_stderr);
 	if (cha_stderr == stderr)
@@ -343,10 +340,10 @@ void cha_exit(status, format, a, b, c, d, e, f, g, h)
     }
 }
 
-void cha_exit_file(status, format, a, b, c, d, e, f, g, h)
-    int status;
-    char *format, *a, *b, *c, *d, *e, *f, *g, *h;
+void cha_exit_file(int status, char *format, ...)
 {
+    va_list ap;
+
     if (Cha_errno)
       return;
 
@@ -358,14 +355,16 @@ void cha_exit_file(status, format, a, b, c, d, e, f, g, h)
     if (progpath)
       fprintf(cha_stderr, "%s: ", progpath);
 
-    if (LineNo == 0)
+    if (Cha_lineno == 0)
       ; /* do nothing */
-    else if (LineNo == LineNoForError)
-      fprintf(cha_stderr, "%s:%d: ", filepath, LineNo);
+    else if (Cha_lineno == Cha_lineno_error)
+      fprintf(cha_stderr, "%s:%d: ", filepath, Cha_lineno);
     else
-      fprintf(cha_stderr, "%s:%d-%d: ", filepath, LineNoForError, LineNo);
+      fprintf(cha_stderr, "%s:%d-%d: ", filepath, Cha_lineno_error, Cha_lineno);
 
-    fprintf(cha_stderr, format, a, b, c, d, e, f, g, h);
+    va_start(ap, format);
+    vfprintf(cha_stderr, format, ap);
+    va_end(ap);
 
     if (status >= 0) {
 	fputc('\n', cha_stderr); 
@@ -389,7 +388,7 @@ void cha_exit_perror(s)
     exit(1);
 }
 
-FILE *fopen_chasenrc()
+FILE *cha_fopen_rcfile()
 {
     FILE *fp;
     char *home_dir, *rc_env, *getenv();
@@ -422,15 +421,8 @@ FILE *fopen_chasenrc()
 	sprintf(chasenrc_path, "%s%s", home_dir, RCFILE);
 	if ((fp = cha_fopen(chasenrc_path, "r", -1)) != NULL)
 	  return fp;
-
-#ifndef VGRAM
-	/* .jumanrc */
-	sprintf(chasenrc_path, "%s%s", home_dir, OLDRCFILE);
-	if ((fp = cha_fopen(chasenrc_path, "r", -1)) != NULL)
-	  return fp;
-#endif
     }
-#ifdef _WIN32
+#ifdef PATHTYPE_MSDOS
     else if ((home_dir = getenv("HOMEDRIVE")) != NULL) {
 	sprintf(chasenrc_path, "%s%s%s", home_dir, getenv("HOMEPATH"), RC2FILE);
 	if ((fp = cha_fopen(chasenrc_path, "r", -1)) != NULL)
@@ -441,14 +433,13 @@ FILE *fopen_chasenrc()
     }
 #endif
 
-
-#ifdef _WIN32
+#ifdef PATHTYPE_MSDOS
     strcpy(chasenrc_path, progpath);
-    sprintf(strrchr(chasenrc_path, '\\'), "\\dic%s", RC2FILE);
+    sprintf(strrchr(chasenrc_path, PATH_DELIMITER) + 1, "dic%s", RC2FILE);
     if ((fp = cha_fopen(chasenrc_path, "r", -1)) != NULL)
       return fp;
     strcpy(chasenrc_path, progpath);
-    sprintf(strrchr(chasenrc_path, '\\'), "\\dic%s", RCFILE);
+    sprintf(strrchr(chasenrc_path, PATH_DELIMITER) + 1, "dic%s", RCFILE);
     if ((fp = cha_fopen(chasenrc_path, "r", -1)) != NULL)
       return fp;
 #endif
@@ -458,7 +449,7 @@ FILE *fopen_chasenrc()
     if ((fp = cha_fopen(chasenrc_path, "r", -1)) != NULL)
       return fp;
 
-#ifdef _WIN32
+#ifdef PATHTYPE_MSDOS
     cha_exit(1, "can't open chasenrc or %s", chasenrc_path);
 #else
     cha_exit(1, "can't open .chasenrc, .jumanrc, or %s", chasenrc_path);
@@ -471,19 +462,19 @@ FILE *fopen_chasenrc()
 /*
  * read .chasenrc and set grammar directory
  */
-void read_grammar_dir()
+void cha_read_grammar_dir()
 {
     FILE *fp;
-    cell_t *cell;
+    chasen_cell_t *cell;
 
-    fp = fopen_chasenrc();
+    fp = cha_fopen_rcfile();
 
-    while (!s_feof(fp)) {
+    while (!cha_s_feof(fp)) {
 	char *s;
-	cell = s_read(fp);
-	s = s_atom(car(cell));
+	cell = cha_s_read(fp);
+	s = cha_s_atom(cha_car(cell));
 	if (strmatch2(s, JSTR_GRAM_FILE, ESTR_GRAM_FILE)) {
-	    strcpy(grammar_dir, s_atom(car(cdr(cell))));
+	    strcpy(grammar_dir, cha_s_atom(cha_car(cha_cdr(cell))));
 	    s = grammar_dir + strlen(grammar_dir);
 	    if (s[-1] != PATH_DELIMITER) {
 		s[0] = PATH_DELIMITER;
