@@ -1,8 +1,8 @@
 /*
- * jfgets.c - fgets() for japaneses
+ * cha_jfgets.c - fgets() for japaneses
  * 		by k-chinen@is.aist-nara.ac.jp, 1996.
  *
- * $Id: jutils.c,v 1.1 1996/03/11 19:43:28 k-chinen Exp k-chinen $
+ * $Id: jfgets.c,v 1.2 2000/11/06 07:28:55 akira-k Exp $
  * 
  * NOTE: An idea of these procedures are taken from youhcan's jutils.c
  *	 for wais-japanese
@@ -15,19 +15,13 @@
 #include <stdio.h>
 #include <string.h>
 
-
-#define DELIMIT2CHARS ""
-#define DELIMIT1CHARS ",."
-
-
-
 #define INNER_BUFSIZE   8192
 
-/* delimiter for jfgets() */
-static char jfgets_delimiter[256] = ""; /*"";*/
+/* delimiter for cha_jfgets() */
+static char jfgets_delimiter[256] = "";
 
 void
-set_jfgets_delimiter(delimiter)
+cha_set_jfgets_delimiter(delimiter)
     char *delimiter;
 {
     strncpy(jfgets_delimiter, delimiter, sizeof(jfgets_delimiter));
@@ -77,7 +71,7 @@ cha_jistoeuc(ibuffer, obuffer)
 {
   unsigned char *p, *o;
   int level, flag;
-  extern int Lang_e;
+  extern int Cha_lang_e;
 
   level = 0;
   flag = 0;
@@ -109,7 +103,7 @@ cha_jistoeuc(ibuffer, obuffer)
     }
     /* ASCII mode or control character in KANJI mode */
 #if 1
-    else if (Lang_e && (*p == ' ' || *p == '\t')) {
+    else if (Cha_lang_e && (*p == ' ' || *p == '\t')) {
 	if (o == obuffer || o[-1] != ' ')
 	  *o++ = ' ';
     }
@@ -135,7 +129,7 @@ cha_jistoeuc(ibuffer, obuffer)
  *      [ASCII WHITES KANJI] -> [ASCII SPACE KANJI]
  */
 
-#if 1
+#if 0
 static
 void
 reduce_white(orig)
@@ -158,63 +152,6 @@ reduce_white(orig)
 }
 #endif
 
-#if 0
-static
-void
-reduce_white(orig)
-     unsigned char *orig;
-{
-  unsigned char *base, *last;
-  unsigned char *first, *cur;
-
-#if 0
-  fprintf(stderr, "*before '%s'\n", orig);
-#endif
-
-  last = NULL;
-  base = orig;
-  while(*base) {
-    if(*base==' '||*base=='\t') {
-      cur = base;
-      cur++;
-      while(*cur==' '||*cur=='\t') {
-        cur++;
-      }
-
-      first = base;
-      if(last!=NULL) {
-        if(*cur>= 0xa0 && *last >= 0xa0) {
-        }
-        else {
-          *first = ' ';
-          first++;
-          /*
-          base++;
-          */
-        }
-      }
-#if 0
-      fprintf(stderr, " change base %4d first %4d cur %4d last '%c'\n",
-                base-orig, first-orig, cur-orig, last==NULL ? 'X' : *last );
-#endif
-      memcpy(first, cur, strlen(cur)+1);
-#if 0
-      fprintf(stderr, "*change '%s'\n", orig);
-#endif
-    }
-
-    last = base;
-    base++;
-  }
-
-#if 0
-  fprintf(stderr, "*after  '%s'\n", orig);
-  fflush(stderr);
-#endif
-}
-#endif
-
-
 /*
  * isterminator - check it is terminator or not
  *
@@ -257,9 +194,6 @@ isterminator(target, termlist)
  *
  */
 
-static unsigned char ibuf[INNER_BUFSIZE];
-static unsigned char *pos=(unsigned char *)""; /* set to the end of line */
-
 static int iskanji1(str, idx)
     unsigned char *str;
     int idx;
@@ -273,9 +207,9 @@ static int iskanji1(str, idx)
 }
 
 /*
- * fget_line - get line via fgets(). So it is really reading function :-)
+ * cha_fget_line - get line via fgets(). So it is really reading function :-)
  */
-char *fget_line(buffer, bufsize, stream)
+char *cha_fget_line(buffer, bufsize, stream)
      char *buffer;
      int bufsize;
      FILE *stream;
@@ -286,13 +220,9 @@ char *fget_line(buffer, bufsize, stream)
   static unsigned char tmp_buf[INNER_BUFSIZE], *tmp, kanji1;
   int last;
 
-  pos = (unsigned char *)buffer;
-
 #if 0
   if(feof(stream)) {
     buffer[0] = '\0';
-    pos = NULL;
-
     return NULL;
   }
 #endif
@@ -303,11 +233,8 @@ char *fget_line(buffer, bufsize, stream)
       kanji1 = 0;
   }
 
-  if(fgets(tmp, bufsize, stream) == NULL) {
-    buffer[0] = '\0';
-    pos = NULL;
+  if(fgets(tmp, bufsize, stream) == NULL)
     return NULL;
-  }
 
   /* remove the last extra character */
   last = strlen(tmp_buf) - 1;
@@ -361,16 +288,18 @@ char *fget_line(buffer, bufsize, stream)
 
 
 /*
- * jfgets - fgets() for Japanese Text.
+ * cha_jfgets - fgets() for Japanese Text.
  *
  */
 
 char *
-jfgets(buffer, bufsize, stream)
+cha_jfgets(buffer, bufsize, stream)
      char *buffer;
      int bufsize;
      FILE *stream;
 {
+  static unsigned char ibuf[INNER_BUFSIZE];
+  static unsigned char *pos=(unsigned char *)""; /* set to the end of line */
   unsigned char *q;
   int count;
   int kflag;            /* kanji flag(0=not found, 1=found) */
@@ -384,11 +313,12 @@ jfgets(buffer, bufsize, stream)
 
   kflag = 0;
   q = (unsigned char *)buffer;
+  bufsize--;
 
   for (count = bufsize; count > 0; count--) {
     /* line is end without '\n', long string read more */
     if(*pos == '\0')
-      if(fget_line(ibuf, sizeof(ibuf), stream) == NULL)
+      if((pos = cha_fget_line(ibuf, sizeof(ibuf), stream)) == NULL)
 	break;
 
     /* KANJI */
@@ -397,6 +327,8 @@ jfgets(buffer, bufsize, stream)
       fprintf(stderr, "@@");
       fflush(stderr);
 #endif
+      if (count<2)
+	break;
       kflag = 1;
       count--;
       *q++ = *pos++;
@@ -404,10 +336,8 @@ jfgets(buffer, bufsize, stream)
 
       /* hit delimiter */
       if(isterminator(pos - 2, jfgets_delimiter)) {
-#if 0
-        fprintf(stderr, "!");
-        fflush(stderr);
-#endif
+	if (*pos == '\n')
+	  pos++;
         break;
       }
     }
@@ -415,7 +345,7 @@ jfgets(buffer, bufsize, stream)
     else {
       /* line is end */
       if(*pos == '\n') {
-        if(fget_line(ibuf, sizeof(ibuf), stream) == NULL)
+        if((pos = cha_fget_line(ibuf, sizeof(ibuf), stream)) == NULL)
 	  break;
 
         /* not have kanji or no space, return with this line */
@@ -426,21 +356,21 @@ jfgets(buffer, bufsize, stream)
 	/* double '\n' is paragraph end. so it is delimiter */
 	if(*pos=='\n')
 	  break;
-
-	/* If the 1st character is not KANJI, insert a space */
-	if(*pos < 0xa0)
-	  *q++ = ' ';
       }
       else {
 #if 0
         fprintf(stderr, "_");
         fflush(stderr);
 #endif
+	kflag = 0;
         *q++ = *pos++;
 
 	/* hit delimiter */
-	if(isterminator(pos - 1, jfgets_delimiter))
+	if(isterminator(pos - 1, jfgets_delimiter)) {
+	  if (*pos == '\n')
+	    pos++;
 	  break;
+        }
       }
     }
 
@@ -451,7 +381,7 @@ jfgets(buffer, bufsize, stream)
 #if 0
   fprintf(stderr, "\n1[%s]\n", buffer);
 #endif
-#if 1
+#if 0
   reduce_white(buffer);
 #endif
 #if 0
