@@ -27,11 +27,10 @@
 
 #define is_spc(c)    ((c)==' '||(c)=='\t')
 
-mrph2_t *Mrph = NULL;
-path_t *Path = NULL;
-int Path_num;
+mrph2_t *Cha_mrph = NULL;
+path_t *Cha_path = NULL;
+int Cha_path_num;
 
-extern char *sufdic_file[MAX_DIC_NUMBER];
 
 /***********************************************************************
  * malloc_chars
@@ -129,13 +128,13 @@ static int malloc_free_path(do_free)
     static int nblock = 0;
 
 #if 0
-    printf("# path %d:%d ", nblock, Path_num);
+    printf("# path %d:%d ", nblock, Cha_path_num);
 #endif
 
-    Path = malloc_free_block((void *)Path, &nblock,
-			     sizeof(path_t) * PATH_NUM, do_free);
+    Cha_path = malloc_free_block((void *)Cha_path, &nblock,
+			     sizeof(path_t) * CHA_PATH_NUM, do_free);
 
-    return Path == NULL;
+    return Cha_path == NULL;
 }
 
 #define malloc_mrph()  malloc_free_mrph(0)
@@ -148,10 +147,10 @@ static int malloc_free_mrph(do_free)
 #if 0
     printf("# mrph %d ", nblock);
 #endif
-    Mrph = malloc_free_block((void *)Mrph, &nblock,
+    Cha_mrph = malloc_free_block((void *)Cha_mrph, &nblock,
 			     sizeof(mrph2_t) * MRPH_NUM, do_free);
 
-    return Mrph == NULL;
+    return Cha_mrph == NULL;
 }
 
 #if 0
@@ -226,7 +225,7 @@ static int undef_mrph_len(target)
 #endif
 
 /***********************************************************************
- *  ̤Хåեɲ
+ * register_undef_mrph1 - ̤Хåեɲ
  ***********************************************************************/
 static int register_undef_mrph1(target, mrph_idx, undef_len, no)
     char *target;
@@ -235,7 +234,7 @@ static int register_undef_mrph1(target, mrph_idx, undef_len, no)
 #if 0
     int undef_len;
 #endif
-    mrph2_t *mrph = &Mrph[mrph_idx];
+    mrph2_t *mrph = &Cha_mrph[mrph_idx];
 
 #if 0
     undef_len = undef_mrph_len(target);
@@ -250,21 +249,17 @@ static int register_undef_mrph1(target, mrph_idx, undef_len, no)
 #endif
     mrph->yomi = "";
     mrph->base_length = mrph->length = undef_len;
-#ifdef VGRAM
     mrph->base = "";
     mrph->pron = "";
-#endif
+    mrph->comp = "\n";
 
-    mrph->hinsi = Undef_info[no].hinsi;
-#ifndef VGRAM
-    mrph->bunrui = Undef_info[no].bunrui;
-#endif
-    mrph->con_tbl = Undef_info[no].con_tbl;
+    mrph->hinsi = Cha_undef_info[no].hinsi;
+    mrph->con_tbl = Cha_undef_info[no].con_tbl;
     mrph->ktype = 0;
     mrph->kform = 0;
     mrph->is_undef = no + 1; /* ̤ */
     mrph->weight = MRPH_DEFAULT_WEIGHT;
-    mrph->info = ""; /* ̣϶ʸȤ롥 */
+    mrph->info = ""; /* ղþ϶ʸȤ롥 */
 
     if (++mrph_idx % MRPH_NUM == 0 && malloc_mrph())
       return FALSE;
@@ -279,7 +274,7 @@ static int register_undef_mrph(target, mrph_idx, undef_len)
 {
     int no;
 
-    for (no = 0; no < Undef_info_num; no++)
+    for (no = 0; no < Cha_undef_info_num; no++)
       if (register_undef_mrph1(target, mrph_idx+no, undef_len, no) == FALSE)
 	return FALSE;
 
@@ -288,171 +283,66 @@ static int register_undef_mrph(target, mrph_idx, undef_len)
 #endif
 
 /*
-------------------------------------------------------------------------------
-        PROCEDURE: <take_data>                  >>> added by T.Nakamura <<<
-------------------------------------------------------------------------------
-*/
-#ifdef VGRAM
-#define DELI_MIDASI '\0'
-#define DELI_YOMI '\0'
-#else
-#define DELI_MIDASI '\t'
-#define DELI_YOMI ' '
-#endif
-void take_data(mrph, pbuf, target)
-    mrph2_t *mrph;
-    char *pbuf, *target;
-{
-    char *p, *s;
-    int  i;
-
-    p = pbuf;
-
-    /* ФĹ */
-    for (i = 0; *p++ != DELI_MIDASI; i++);
-    mrph->base_length = mrph->length = i;
-    /* ʻʬ No. */
-    for (i = 0; *p != DELI_YOMI; i = i * 10 + *p++ - '0');
-    mrph->hinsi = i; p++;
-#ifndef VGRAM
-    /* ʻʬ No. */
-    for (i = 0; *p != DELI_YOMI; i = i * 10 + *p++ - '0');
-    mrph->bunrui = i; p++;
-#endif
-    /* ѷ No. */
-    for (i = 0; *p != DELI_YOMI; i = i * 10 + *p++ - '0');
-    mrph->ktype = i; p++;
-    /* ѷ No. */
-    for (i = 0; *p != DELI_YOMI; i = i * 10 + *p++ - '0');
-    mrph->kform = i; p++;
-    /* Ť */
-#ifndef VGRAM
-    if (*p < '0' || *p > '9')
-      cha_exit(1, "The format of the patricia dictionary is invalid.");
-#endif
-    for (i = 0; *p != DELI_YOMI; i = i * 10 + *p++ - '0');
-    mrph->weight = i; p++;
-    /* ɤ */
-    mrph->yomi = p;
-    p = strchr(p, DELI_YOMI) + 1;
-#ifdef VGRAM
-    /* ȯ */
-    mrph->pron = p;
-    p += strlen(p) + 1;
-    /*  */
-    mrph->base = p;
-    p += strlen(p) + 1;
-    /* ³ơֹ֥ */
-    if (*p < '0' || *p > '9')
-      cha_exit(1, "The format of the patricia dictionary is invalid.");
-#endif
-    for (i = 0; *p != DELI_YOMI; i = i * 10 + *p++ - '0');
-    mrph->con_tbl = i; p++;
-    /* ̣ */
-    mrph->info = p;
-
-    mrph->midasi = target;
-    mrph->is_undef = 0;
-}
-
-/***********************************************************************
- * compare_top_str1 - compare strings to prefix of s1
- ***********************************************************************/
-static int compare_top_str1(s1, s2)
-    char *s1, *s2;
-{
-    /* 1996.07.28 by akira-k */
-    for (;;) {
-	if (*s1 == '\0') return TRUE;
-	if (*s1++ != *s2++) return FALSE;
-    }
-}
-
-/***********************************************************************
- * katuyou_process - find all forms which match the follow string
- ***********************************************************************/
-static int *katuyou_process(follows, ktype)
-    char *follows;
-    int ktype;
-{
-    static int formstr[FORM_NO];
-    int *fs = formstr;
-    int f;
-
-    for (f = 1; Form[ktype][f].name; f++)
-      if (compare_top_str1(Form[ktype][f].gobi, follows))
-	*fs++ = f;
-    *fs = 0;
-
-    return formstr;
-}
-
-/***********************************************************************
- * register_mrph - ѤĴ٤ʤ1ĤηǤХåեɲ
+ * register_mrph - ѤĴ٤ʤǤХåեɲ
  *
- * mrph_nump: ХåեɲäǤο
- ************************************************************************/
-static int register_mrph(mrph_idx, nmrphp)
-    int mrph_idx, *nmrphp;
+ * Retern value:
+ * If successful, this rutine returns the number of morphs
+ * added to the buffer. If an error occurs, return -1.
+ */
+static int register_mrph(int mrph_idx)
 {
     int new_mrph_idx = mrph_idx;
-    mrph2_t *new_mrph = &Mrph[mrph_idx];
+    mrph2_t *new_mrph = &Cha_mrph[mrph_idx];
 
-    if (
-#ifdef VGRAM
-	Hinsi[new_mrph->hinsi].kt == 1
-#else
-	Class[new_mrph->hinsi][new_mrph->bunrui].kt
-#endif
-	) {
+    if (!new_mrph->ktype) {
+	/* Ѥʤ */
+	if (++new_mrph_idx % MRPH_NUM == 0 && malloc_mrph())
+	  return -1;
+    } else {
 	/* Ѥ */
-	if (new_mrph->kform == 0) {
-	    /* 촴 */
-	    int *f;
-	    int ktype = new_mrph->ktype;
-	    int length = new_mrph->length;
-	    int con_tbl = new_mrph->con_tbl;
-	    char *follows = new_mrph->midasi + new_mrph->base_length;
-	    int new_mrph_idx0 = new_mrph_idx;
-	    for (f = katuyou_process(follows, ktype); *f; f++) {
-		if (new_mrph_idx != new_mrph_idx0)
-		  *new_mrph = Mrph[new_mrph_idx0];
-		new_mrph->kform = *f;
-		new_mrph->length = length + strlen(Form[ktype][*f].gobi);
-		new_mrph->con_tbl = con_tbl + *f - 1;
-		if (++new_mrph_idx % MRPH_NUM == 0 && malloc_mrph())
-		  return FALSE;
-		new_mrph = &Mrph[new_mrph_idx];
-	    }
-	} else {
+	if (new_mrph->kform) {
 	    /* 촴ʤ */
 	    new_mrph->base_length = 0;
 	    new_mrph->yomi = "";
-#ifdef VGRAM
 	    new_mrph->pron = "";
-#endif
 	    if (++new_mrph_idx % MRPH_NUM == 0 && malloc_mrph())
-	      return FALSE;
+	      return -1;
+	} else {
+	    /* 촴 */
+	    int f;
+	    int ktype = new_mrph->ktype;
+	    int baselen = new_mrph->length;
+	    int con_tbl = new_mrph->con_tbl;
+	    char *follows = new_mrph->midasi + baselen;
+	    int new_mrph_idx0 = new_mrph_idx;
+	    for (f = 1; Cha_form[ktype][f].name; f++) {
+		if (!Cha_form[ktype][f].gobi[0] ||
+		    (follows[0] == Cha_form[ktype][f].gobi[0] &&
+		     !memcmp(follows, Cha_form[ktype][f].gobi, Cha_form[ktype][f].gobi_len))) {
+		    if (new_mrph_idx != new_mrph_idx0)
+		      *new_mrph = Cha_mrph[new_mrph_idx0];
+		    new_mrph->kform = f;
+		    new_mrph->length = baselen + Cha_form[ktype][f].gobi_len;
+		    new_mrph->con_tbl = con_tbl + f - 1;
+		    if (++new_mrph_idx % MRPH_NUM == 0 && malloc_mrph())
+		      return -1;
+		    new_mrph = &Cha_mrph[new_mrph_idx];
+		}
+	    }
 	}
-    } else {                         /* Ѥʤ */
-	if (++new_mrph_idx % MRPH_NUM == 0 && malloc_mrph())
-	  return FALSE;
     }
 
-    *nmrphp = new_mrph_idx - mrph_idx;
-
-    return TRUE;
+    return new_mrph_idx - mrph_idx;
 }
 
-/***********************************************************************
- * convert_pat_mrphs - ѤĴ٤ʤǤХåեɲ
- *
- * mrph_nump: ХåեɲäǤο
- ***********************************************************************/
-static int convert_pat_mrphs(target, dic_buffer, mrph_idx, mrph_nump)
-    char *target;
-    char **dic_buffer;
-    int mrph_idx, *mrph_nump;
+/*
+ * convert_mrphs - ǤХåեɲ
+ * 
+ * Retern value:
+ * If successful, this rutine returns the number of morphs
+ * added to the buffer. If an error occurs, return -1.
+ */
+static int convert_mrphs(char *target, char **dic_buffer, int mrph_idx)
 {
     int nmrph;
     int new_mrph_idx = mrph_idx;
@@ -462,86 +352,27 @@ static int convert_pat_mrphs(target, dic_buffer, mrph_idx, mrph_nump)
 #if 0
 	fprintf(stdout, "line: %s\n", *pbuf); fflush(stdout);
 #endif
-	take_data(&Mrph[new_mrph_idx], *pbuf, target);
-	if (register_mrph(new_mrph_idx, &nmrph) == FALSE)
-	  return FALSE;
+	cha_get_mrph_data(&Cha_mrph[new_mrph_idx], *pbuf, target);
+#if 0
+	printf("%s %d %d %d\n",
+	       Cha_mrph[new_mrph_idx].yomi,
+	       Cha_mrph[new_mrph_idx].hinsi,
+	       Cha_mrph[new_mrph_idx].ktype,
+	       Cha_mrph[new_mrph_idx].kform);
+#endif
+	nmrph = register_mrph(new_mrph_idx);
+	if (nmrph < 0)
+	  return -1;
 	new_mrph_idx += nmrph;
     }
-    *mrph_nump = new_mrph_idx - mrph_idx;
 
-    return TRUE;
+    return new_mrph_idx - mrph_idx;
 }
 
-/***********************************************************************
- * convert_suf_mrphs - ѤĴ٤ʤǤХåեɲ
- *
- * mrph_nump: ХåեɲäǤο
- ***********************************************************************/
-static int convert_suf_mrphs(target, dic_buffer, file, mrph_idx, mrph_nump)
-    char *target;
-    char *file;
-    long *dic_buffer;
-    int mrph_idx, *mrph_nump;
-{
-    int nmrph;
-    int new_mrph_idx = mrph_idx;
-    int i, nsuf = (int)dic_buffer[0];
-
-    for (i = 1; i <= nsuf; i++) {
-#if 0
-	char *line = (char *)sa_getline(file, dic_buffer[i]);
-#endif
-	char *line = (char *)(((SUFARY *)file)->txtmap + dic_buffer[i]);
-	if (0) {
-	  char s[1024];
-	  memset(s,0,1024);
-	  memccpy(s,line,'\n',256);
-	  printf("line: %s",s);fflush(stdout);
-	}
-	take_data(&Mrph[new_mrph_idx], line, target);
-	if (register_mrph(new_mrph_idx, &nmrph) == FALSE)
-	  return FALSE;
-	new_mrph_idx += nmrph;
-    }
-    *mrph_nump = new_mrph_idx - mrph_idx;
-
-    return TRUE;
-}
-
-#if 0
-/*
-------------------------------------------------------------------------------
-  PROCEDURE: <trim_space> ڡκ (ȾѥڡΤߤѹ by T.U
-                                                                   '96.01.11)
-------------------------------------------------------------------------------
-*/
-static int trim_space(str, pos)
-    char *str;
-    int pos;
-{
-    for (;;) {
-	/* Ⱦѥڡ */
-	if (str[pos] == ' ')
-	  pos++;
-#if 0
-	/* ѥڡ */
-	else if ((unsigned char)str[pos  ] == 0xA1 &&
-		 (unsigned char)str[pos+1] == 0xA1)
-	  pos += 2;
-#endif
-	else
-	  break;
-    }
-    return pos;
-}
-#endif
-
 /*
-------------------------------------------------------------------------------
-  PROCEDURE: <pos_match_process>
-------------------------------------------------------------------------------
-*/
-static int pos_match_process(pos, p_idx)
+ * collect_mrphs_for_pos()
+ */
+static int collect_mrphs_for_pos(pos, p_idx)
     int pos, *p_idx;
 {
     static int p_start;
@@ -553,11 +384,11 @@ static int pos_match_process(pos, p_idx)
 	p_idx[j++] = 0;
 	p_start = 1;
     } else {
-	for (i = p_start; i < Path_num; i++) {
-	    if (Path[i].end <= pos) {
+	for (i = p_start; i < Cha_path_num; i++) {
+	    if (Cha_path[i].end <= pos) {
 		if (i == p_start)
 		  p_start++;
-		if (Path[i].end == pos)
+		if (Cha_path[i].end == pos)
 		  p_idx[j++] = i;
 	    }
 	}
@@ -567,11 +398,9 @@ static int pos_match_process(pos, p_idx)
     return j;
 }
 
-/*
-------------------------------------------------------------------------------
-  PROCEDURE: <check_connect>
-------------------------------------------------------------------------------
-*/
+/***********************************************************************
+ * check_connect()
+ ***********************************************************************/
 static int check_connect(pos, m_num, p_idx)
     int pos, m_num, *p_idx;
 {
@@ -585,53 +414,57 @@ static int check_connect(pos, m_num, p_idx)
 	int   pno[PATH1_NUM];
     } path_cost_t;
 
+/*
+    static short best_start, best_end, best_state;
+    static int best_cost;
+*/
     static path_cost_t pcost[PATH1_NUM];
     int pcost_num;
-    int path[PATH1_NUM], *new_path;
     mrph2_t *new_mrph;
-    int i, pno, pcostno, npath;
+    int i, pno, pcostno;
     int	haba_cost, con_cost, cost, mrph_cost;
     int con_tbl, next_state;
 
 #ifdef DEBUG
     printf("[m:%d] ", m_num);
 #endif
-    new_mrph = &Mrph[m_num];
+    new_mrph = &Cha_mrph[m_num];
     con_tbl = new_mrph->con_tbl;
 
     pcost[0].state = -1;
     pcost_num = 0;
 
     for (i = 0; (pno = p_idx[i]) >= 0; i++) {
-	/* ȥޥȥĴ٤Ƽ֤³ȤФ */
-	next_state = check_automaton
-	  (Path[pno].state,
-#ifdef KOCHA
-	   Path[Path[pno].path[0]].state,
-	   Mrph[Path[pno].mrph_p].midasi,
-	   Mrph[Path[pno].mrph_p].is_undef,
+#if 0
+	if (i>0&&pcost[0].min_cost<Cha_path[pno].cost)
+	  continue;
 #endif
-	   con_tbl, Con_cost_undef, &con_cost);
+	/* ȥޥȥĴ٤Ƽ֤³ȤФ */
+	next_state = cha_check_automaton
+	  (Cha_path[pno].state,
+#ifdef KOCHA
+	   Cha_path[Cha_path[pno].path[0]].state,
+	   Cha_mrph[Cha_path[pno].mrph_p].midasi,
+	   Cha_mrph[Cha_path[pno].mrph_p].is_undef,
+#endif
+	   con_tbl, Cha_con_cost_undef, &con_cost);
+
+	if (con_cost == -1) continue;
 
 #ifdef DEBUG
-	printf("[m1:%d,m2:%d,cost:%d,state:%d]\n",
-	       Path[pno].mrph_p,m_num,con_cost,next_state);
+	printf("[%3d, %3d, pos:%d, len:%d, state:%5d,%5d, cost:%d, undef:%d]\n",
+	       Cha_path[pno].mrph_p,m_num,pos,new_mrph->length,Cha_path[pno].state,next_state,cost,new_mrph->is_undef);
 #endif
-#ifdef VGRAM
-	if (con_cost == -1) continue;
-#else
-	if (con_cost == 0) continue;
-#endif
-
 	/* cost ׻ */
-	cost = Path[pno].cost + con_cost * Con_cost_weight;
+	cost = Cha_path[pno].cost + con_cost * Cha_con_cost_weight;
 
 	/* ɤ pcost °뤫Ĵ٤ */
 	for (pcostno = 0; pcostno < pcost_num; pcostno++)
 	  if (next_state == pcost[pcostno].state)
 	    break;
 	if (pcostno < pcost_num) {
-	    if (cost - pcost[pcostno].min_cost > Cost_width)
+	    /* tricky: when Cha_cost_width is -1, ">-1" means ">=0" */
+	    if (cost - pcost[pcostno].min_cost > Cha_cost_width)
 	      continue;
 	} else {
 	    /*  pcost  */
@@ -642,13 +475,18 @@ static int check_connect(pos, m_num, p_idx)
 	}
 
 	/* pcost Ͽ */
-	pcost[pcostno].cost[pcost[pcostno].num] = cost;
-	pcost[pcostno].pno[pcost[pcostno].num] = pno;
-	if (cost < pcost[pcostno].min_cost) {
+	if (Cha_cost_width < 0) {
 	    pcost[pcostno].min_cost = cost;
-	    pcost[pcostno].min_cost_no = pcost[pcostno].num;
+	    pcost[pcostno].pno[0] = pno;
+	} else {
+	    pcost[pcostno].cost[pcost[pcostno].num] = cost;
+	    pcost[pcostno].pno[pcost[pcostno].num] = pno;
+	    if (cost < pcost[pcostno].min_cost) {
+		pcost[pcostno].min_cost = cost;
+		pcost[pcostno].min_cost_no = pcost[pcostno].num;
+	    }
+	    pcost[pcostno].num++;
 	}
-	pcost[pcostno].num++;
     }
 
     if (pcost_num == 0)
@@ -656,66 +494,92 @@ static int check_connect(pos, m_num, p_idx)
 
     /* ǥ */
     if (new_mrph->is_undef) {
-	mrph_cost = Undef_info[new_mrph->is_undef-1].cost
-	  + Undef_info[new_mrph->is_undef-1].cost_step * new_mrph->length / 2;
+	mrph_cost = Cha_undef_info[new_mrph->is_undef-1].cost
+	  + Cha_undef_info[new_mrph->is_undef-1].cost_step * new_mrph->length / 2;
     } else {
-#ifdef VGRAM
-	mrph_cost = Hinsi[new_mrph->hinsi].cost;
-#else
-	mrph_cost = Class[new_mrph->hinsi][new_mrph->bunrui].cost;
-#endif
+	mrph_cost = Cha_hinsi[new_mrph->hinsi].cost;
     }
-    mrph_cost *= new_mrph->weight * Mrph_cost_weight;
+    mrph_cost *= new_mrph->weight * Cha_mrph_cost_weight;
 
 #ifdef KOCHA
     if (new_mrph->midasi &&
 	(unsigned char)new_mrph->midasi[0] == 0xA1 &&
 	(unsigned char)new_mrph->midasi[1] == 0xA1) {
 	for (i = 0; i < pcost[0].num; i++) {
-	    new_path = malloc_int(2);
-	    new_path[0] = pcost[0].pno[i];
-	    new_path[1] = -1;
+	    int *new_path;
+	    Cha_path[Cha_path_num].path = malloc_int(2);
+	    Cha_path[Cha_path_num].path[0] = pcost[0].pno[i];
+	    Cha_path[Cha_path_num].path[1] = -1;
 
-	    Path[Path_num].cost = pcost[0].cost[i] + mrph_cost;
-	    Path[Path_num].mrph_p = m_num;
-	    Path[Path_num].state = pcost[0].state;
-	    Path[Path_num].start = pos;
-	    Path[Path_num].end = pos + new_mrph->length;
-	    Path[Path_num].path = new_path;
+	    Cha_path[Cha_path_num].cost = pcost[0].cost[i] + mrph_cost;
+	    Cha_path[Cha_path_num].mrph_p = m_num;
+	    Cha_path[Cha_path_num].state = pcost[0].state;
+	    Cha_path[Cha_path_num].start = pos;
+	    Cha_path[Cha_path_num].end = pos + new_mrph->length;
 #ifdef KOCHA_DEBUG
 	    printf("[Sp:%d,prev:%d,m:%d,c:%d,pc:%d]\n",
-		   Path_num,Path[Path_num].path[0],m_num,pcost[0].cost[i],Path[Path_num].cost);
+		   Cha_path_num,Cha_path[Cha_path_num].path[0],m_num,pcost[0].cost[i],Cha_path[Cha_path_num].cost);
 #endif
-	    if (++Path_num % PATH_NUM == 0 && malloc_path())
+	    if (++Cha_path_num % CHA_PATH_NUM == 0 && malloc_path())
 	      return FALSE;
 	}
     } else {
 #endif /* KOCHA */
+#if 0
+	if (Cha_path[Cha_path_num].end == pos + new_mrph->length &&
+	    Cha_path[Cha_path_num].state == pcost[pcostno].state &&
+	    Cha_path[Cha_path_num].cost > pcost[pcostno].min_cost + mrph_cost) {
+	    return TRUE;
+	}
+#endif
+
 	for (pcostno = 0; pcostno < pcost_num; pcostno++) {
 	    /* ˤޤäƤѥȴФ */
-	    haba_cost = pcost[pcostno].min_cost + Cost_width;
-	    npath = 0;
-	    path[npath++] = pcost[pcostno].pno[pcost[pcostno].min_cost_no];
-	    for (i = 0; i < pcost[pcostno].num; i++)
-	      if (pcost[pcostno].cost[i] <= haba_cost && i != pcost[pcostno].min_cost_no)
-		path[npath++] = pcost[pcostno].pno[i];
-	    path[npath++] = -1;
+	    if (Cha_cost_width < 0) {
+		Cha_path[Cha_path_num].path = malloc_int(2);
+		Cha_path[Cha_path_num].path[0] = pcost[pcostno].pno[0];
+		Cha_path[Cha_path_num].path[1] = -1;
+	    } else {
+		int npath = 0;
+		int path[PATH1_NUM];
+		haba_cost = pcost[pcostno].min_cost + Cha_cost_width;
+		path[npath++] = pcost[pcostno].pno[pcost[pcostno].min_cost_no];
+		for (i = 0; i < pcost[pcostno].num; i++)
+		  if (pcost[pcostno].cost[i] <= haba_cost && i != pcost[pcostno].min_cost_no)
+		    path[npath++] = pcost[pcostno].pno[i];
+		path[npath++] = -1;
+		memcpy(Cha_path[Cha_path_num].path = malloc_int(npath),
+		       path, sizeof(int) * npath);
+	    }
 
-	    new_path = malloc_int(npath);
-	    memcpy(new_path, path, sizeof(int) * npath);
-
-	    /* Path Ͽ */
-	    Path[Path_num].cost = pcost[pcostno].min_cost + mrph_cost;
-	    Path[Path_num].mrph_p = m_num;
-	    Path[Path_num].state = pcost[pcostno].state;
-	    Path[Path_num].start = pos;
-	    Path[Path_num].end = pos + new_mrph->length;
-	    Path[Path_num].path = new_path;
-#ifdef DEBUG
-	    printf("[p:%d,prev:%d,m:%d,c:%d,pc:%d]\n",
-		   Path_num,Path[Path_num].path[0],m_num,pcost[0].cost[i],Path[Path_num].cost);
+	    /* Cha_path Ͽ */
+	    Cha_path[Cha_path_num].cost = pcost[pcostno].min_cost + mrph_cost;
+	    Cha_path[Cha_path_num].mrph_p = m_num;
+	    Cha_path[Cha_path_num].state = pcost[pcostno].state;
+	    Cha_path[Cha_path_num].start = pos;
+	    Cha_path[Cha_path_num].end = pos + new_mrph->length;
+#if 0
+	    if (Cha_path[Cha_path_num].start < best_start ||
+		Cha_path[Cha_path_num].end > best_end ||
+		Cha_path[Cha_path_num].end == best_end &&
+		Cha_path[Cha_path_num].state == best_state &&
+		Cha_path[Cha_path_num].cost < best_cost) {
+		best_start = Cha_path[Cha_path_num].start;
+		best_end = Cha_path[Cha_path_num].end;
+		best_state = Cha_path[Cha_path_num].state;
+		best_cost = Cha_path[Cha_path_num].cost;
+	    }
 #endif
-	    if (++Path_num % PATH_NUM == 0 && malloc_path())
+#if 0
+	    printf("[%4d,%4d,%5d, %5d]\n",Cha_path[Cha_path_num].start,Cha_path[Cha_path_num].end,Cha_path[Cha_path_num].state,Cha_path[Cha_path_num].cost);
+#endif
+#ifdef DEBUG
+	    printf("%3d %3d %5d [p:%d,prev:%d,m:%d,c:%d,pc:%d]\n",
+		   Cha_path[Cha_path_num].start, Cha_path[Cha_path_num].end,
+		   Cha_path[Cha_path_num].state,
+		   Cha_path_num,Cha_path[Cha_path_num].path[0],m_num,pcost[0].cost[i],Cha_path[Cha_path_num].cost);
+#endif
+	    if (++Cha_path_num % CHA_PATH_NUM == 0 && malloc_path())
 	      return FALSE;
 	}
 #ifdef KOCHA
@@ -728,47 +592,37 @@ static void set_mrph_end(mrph)
     mrph2_t *mrph;
 {
     mrph->midasi = mrph->yomi = mrph->info = "";
-#ifdef VGRAM
     mrph->base = mrph->pron = "";
-#endif
+    mrph->comp = "\n";
     mrph->base_length = mrph->length = 3;
 
     mrph->hinsi = 0;
-#ifndef VGRAM
-    mrph->bunrui = 0;
-#endif
-
-    mrph->con_tbl = 0;
     mrph->ktype = 0;
     mrph->kform = 0;
-
+    mrph->con_tbl = 0;
     mrph->is_undef = 0;
     mrph->weight = MRPH_DEFAULT_WEIGHT;
 }
 
-#ifdef VGRAM
 static int set_mrph_bkugiri()
 {
     static int bkugiri_num;
     int h;
     mrph2_t *mrph;
 
-    if (Mrph[1].midasi)
+    if (Cha_mrph[1].midasi)
       return bkugiri_num;
 
-    for (h = 0; Hinsi[h].name; h++) {
-	if (!Hinsi[h].bkugiri)
+    for (h = 0; Cha_hinsi[h].name; h++) {
+	if (!Cha_hinsi[h].bkugiri)
 	  continue;
-	mrph = &Mrph[++bkugiri_num];
+	mrph = &Cha_mrph[++bkugiri_num];
 	/* memset: unnecessary? */
 	memset(mrph, 0, sizeof(mrph2_t));
 
 	mrph->hinsi = h;
-	mrph->con_tbl = check_table_for_undef(h);
-	mrph->midasi = mrph->yomi = mrph->base = Hinsi[h].bkugiri;
-#ifdef VGRAM
-	mrph->pron = mrph->midasi;
-#endif
+	mrph->con_tbl = cha_check_table_for_undef(h);
+	mrph->midasi = mrph->yomi = mrph->base = mrph->pron = Cha_hinsi[h].bkugiri;
 	mrph->info = "";
 #if 0
 	mrph->base_length = mrph->length = 0;
@@ -779,15 +633,14 @@ static int set_mrph_bkugiri()
     }
     return bkugiri_num;
 }
-#endif
 
 static int strcmp_anno(target)
     char *target;
 {
     int i;
 
-    for (i = 1; Anno_info[i].str1; i++)
-      if (!memcmp(target, Anno_info[i].str1, Anno_info[i].len1))
+    for (i = 1; Cha_anno_info[i].str1; i++)
+      if (!memcmp(target, Cha_anno_info[i].str1, Cha_anno_info[i].len1))
 	return -i;
     return 0;
 }
@@ -795,8 +648,9 @@ static int strcmp_anno(target)
 /*
  * check_undefword_len()
  *
- * char_type:
- *   0: 
+ * type: -n(n=idx of Cha_anno_info) / 0(space) / 1(1byte) / 2(2byte) / 3(3byte)
+ * stat: -n(n=idx of Cha_anno_info) / 0(space) / 1(e:[a-zA-Z],j:1byte)
+ *       / 2(katakana) / 3(alphabet) / 4(otherwise)
  */
 static void check_undefword_len(target, undefword_len, char_type, target_len)
     char *target, *char_type;
@@ -805,8 +659,8 @@ static void check_undefword_len(target, undefword_len, char_type, target_len)
     unsigned char *t;
     short *ulen0, *ulen;
     char *type;
-    int stat0 = 1, stat;
-    anno_info *anno;
+    int stat0 = 1, stat = 0;
+    anno_info *anno = NULL;
 
     memset(undefword_len, 0, target_len * sizeof(short));
     memset(char_type, 1, target_len + 1);
@@ -818,8 +672,8 @@ static void check_undefword_len(target, undefword_len, char_type, target_len)
     while ((char *)t < target + target_len) {
 	if (stat0 < 0 &&
 	    (anno->len2 == 0 ||
-	     (char *)t - target > anno->len2 &&
-	     !memcmp(t - anno->len2, anno->str2, anno->len2)))
+	     ((char *)t - target > anno->len2 &&
+	     !memcmp(t - anno->len2, anno->str2, anno->len2))))
 	  stat0 = 99;
 	if (stat0 < 0) {
 	    ;
@@ -827,17 +681,25 @@ static void check_undefword_len(target, undefword_len, char_type, target_len)
 	    *type = stat = 0;
 	} else if ((stat = strcmp_anno(t)) < 0) {
 	    *type = stat;
-	    anno = &Anno_info[-stat];
-	} else if (Lang_e
-		   ? (*t>='a' && *t<='z' || *t>='A' && *t<='Z')
+	    anno = &Cha_anno_info[-stat];
+	} else if (Cha_lang_e
+		   ? ((*t>='a' && *t<='z') || (*t>='A' && *t<='Z'))
 		   : !(*t & 0x80)) {
 	    /*: (!(*t & 0x80) && !(*t>='0'&&*t<='9'||*t=='.'||*t==','))) {*/
 	    /* [a-zA-Z] / [^1-9.,] */
 	    stat = 1;
-	} else if (t[0] == 0xa5 && t[1] >= 0xa1 ||
-		   t[0] == 0xa1 && t[1] == 0xbc) {
-	    /* 0xa5a1-0xa5ff: zenkaku katakana */
-	    /* 0xa1bc: zenkaku chou-on */
+	} else if (/* znekaku chou-on */
+		   (stat0 == 2 && t[0] == 0xa1 && t[1] == 0xbc) ||
+		   /* 0xa5a1-0xa5ff: zenkaku katakana */
+		   (t[0] == 0xa5 && t[1] >= 0xa1 &&
+		   !(stat0 != 2 &&
+		     /* small aiueo,tsu,yayuyo,wa and chou-on */
+		     ((t[0] == 0xa5 &&
+		       (t[1] == 0xa1 || t[1] == 0xa3 || t[1] == 0xa5 ||
+			t[1] == 0xa7 || t[1] == 0xa9 || t[1] == 0xc3 ||
+			t[1] == 0xe3 || t[1] == 0xe5 || t[1] == 0xe7 ||
+			t[1] == 0xee)) ||
+		      (t[0] == 0xa1 && t[1] == 0xbc))))) {
 	    stat = 2;
 	} else if (t[0] == 0xa3 && t[1] >= 0xc1) {
 	    /* 0xa3c1-0xa3ff: zenkaku alphabet */
@@ -859,6 +721,9 @@ static void check_undefword_len(target, undefword_len, char_type, target_len)
 	    t += len;
 	    ulen += len;
 	    type += len;
+	} else if (t[0] == 0x8f && (t[1] & 0x80) && (t[2] & 0x80)) {
+	    *type = 3;
+	    t += 3; ulen += 3, type += 3;
 	} else if ((t[0] & 0x80) && (t[1] & 0x80)) {
 	    *type = 2;
 	    t += 2; ulen += 2, type += 2;
@@ -872,18 +737,17 @@ static void check_undefword_len(target, undefword_len, char_type, target_len)
 }
 
 /***********************************************************************
- * chasen_sent() - ʸǲϤ
+ * cha_parse_sentence() - ʸǲϤ
  *
  * return value:
  *     0 - ok
  *     1 - no result / too many morphs
  ***********************************************************************/
-int chasen_sent(target, target_len, opt_nobk)
+int cha_parse_sentence(target, target_len, opt_nobk)
     char *target;
     int target_len; /* should be >0 */
     int opt_nobk;
 {
-    extern int num_sufdic_file;
     int  i, dic_no;
     int  pos, pos_end;
     int  path_idx[PATH1_NUM], path_idx_num;
@@ -891,8 +755,7 @@ int chasen_sent(target, target_len, opt_nobk)
     int  undef_len;
     int  bkugiri_num = 0, bk;
     static int path0 = -1;
-    long *sufdic_buffer;
-    char *patdic_buffer[256];
+    char *dic_buffer[256];
     static short undefword_len[CHA_INPUT_SIZE];
     static char char_type[CHA_INPUT_SIZE];
 
@@ -904,22 +767,23 @@ int chasen_sent(target, target_len, opt_nobk)
     free_mrph();
 
     /* ʸƬ */
-    Path[0].start = Path[0].end = 0;
-    Path[0].path = &path0;
-    Path[0].cost = 0;
-    Path[0].mrph_p = 0;
-    Path[0].state = 0;
+    Cha_path[0].start = Cha_path[0].end = 0;
+    Cha_path[0].path = &path0;
+    Cha_path[0].cost = 0;
+    Cha_path[0].mrph_p = 0;
+    Cha_path[0].state = 0;
 
-    Path_num = 1;
-    set_mrph_end(&Mrph[0]);
-#ifdef VGRAM
+    Cha_path_num = 1;
+    set_mrph_end(&Cha_mrph[0]);
     if (!opt_nobk)
       bkugiri_num = set_mrph_bkugiri();
-#endif
     new_mrph_idx = mrph_idx = bkugiri_num + 1;
 
     for (pos = pos_end = 0; pos < target_len;
-	 pos += char_type[pos] == 2 ? 2 : undefword_len[pos],
+	 pos += char_type[pos] == 2 &&
+	 !((unsigned char)target[pos] == 0xa3 &&
+	 (unsigned char)target[pos+1] >= 0xc1)
+	 ? 2 : undefword_len[pos],
 	 pos_end = pos) {
 #if 0
 	printf("# mrph %d\n", mrph_idx);
@@ -930,7 +794,7 @@ int chasen_sent(target, target_len, opt_nobk)
 	if (pos == target_len)
 	  break;
 
-	path_idx_num = pos_match_process(pos_end, path_idx);
+	path_idx_num = collect_mrphs_for_pos(pos_end, path_idx);
 #if 0
 	printf("# path_idx_num %d\n", path_idx_num);
 #endif
@@ -939,63 +803,61 @@ int chasen_sent(target, target_len, opt_nobk)
 
 #if 1
 	for (bk = 0; bk < bkugiri_num; bk++) {
-	    int path_num0, path_num1;
-	    path_num0 = Path_num;
+	    int path_num;
+	    path_num = Cha_path_num;
 	    /* ʸڤɲ */
 	    if (check_connect(pos, bk + 1, path_idx) == FALSE)
 	      goto error_end;
 #if 0
-	    printf("PATH: %d: %d -> %d\n", pos, path_num0, Path_num);
+	    printf("PATH: %d: %d -> %d\n", pos, path_num, Cha_path_num);
 #endif
 	    /* ɲä줿 path  path_idx ɲ */
-	    if (Path_num > path_num0)
-	      for (; path_num0 < Path_num; path_num0++)
-		path_idx[path_idx_num++] = path_num0;
+	    if (Cha_path_num > path_num)
+	      for (; path_num < Cha_path_num; path_num++)
+		path_idx[path_idx_num++] = path_num;
 	    path_idx[path_idx_num] = -1;
 	}
 #endif
 
 	/* (ʸΤ߸) */
 	if (char_type[pos] == 2) {
-	    for (dic_no = 0; dic_no < number_of_tree; dic_no++) {
+	    for (dic_no = 0; dic_no < Pat_ndicfile; dic_no++) {
 		int nmrph;
 		/* ѥȥꥷڤǤ򸡺 */
-		pat_search(dic_file[dic_no], target + pos,
- 			   &tree_top[dic_no], patdic_buffer);
-
-		/* ѤĤķǤ Mrph ɲ */
-		if (convert_pat_mrphs(target + pos, patdic_buffer,
-				      new_mrph_idx, &nmrph) == FALSE)
+		pat_search(Pat_dicfile[dic_no], target + pos,
+			   dic_buffer);
+		/* ѤĤķǤ Cha_mrph ɲ */
+		nmrph = convert_mrphs(target + pos, dic_buffer,
+				      new_mrph_idx);
+		if (nmrph < 0)
 		  goto error_end;
 		new_mrph_idx += nmrph;
 	    }
 	}
 
-	for (dic_no = 0; dic_no < num_sufdic_file; dic_no++) {
-	  int nmrph;
-	  /* SUFARY ե뤫Ǥ򸡺 */
-	  sa_reset(sufdic_file[dic_no]);
-	  sufdic_buffer =
-	    (long *)sa_common_prefix_search(sufdic_file[dic_no],
-					    target + pos, '\0');
-	  /* ѤĤķǤ Mrph ɲ */
-	  if (convert_suf_mrphs(target + pos, sufdic_buffer,
-				sufdic_file[dic_no], new_mrph_idx, &nmrph)
-	      == FALSE)
-	    goto error_end;
-	  free(sufdic_buffer);
-	  new_mrph_idx += nmrph;
+	for (dic_no = 0; dic_no < Suf_ndicfile; dic_no++) {
+	    int nmrph;
+	    /* SUFARY ե뤫Ǥ򸡺 */
+	    sa_common_prefix_search(Suf_dicfile[dic_no],
+				    target + pos,
+				    dic_buffer);
+	    /* ѤĤķǤ Cha_mrph ɲ */
+	    nmrph = convert_mrphs(target + pos, dic_buffer,
+				  new_mrph_idx);
+	    if (nmrph < 0)
+		goto error_end;
+	    new_mrph_idx += nmrph;
 	}
 
 #if 0
 	/*
 	 * ̤ϢܥȤ 0 ΤȤñ줬1Ĥ⼭Ǥʤä
-	 * ̤ Mrph ɲ
+	 * ̤ Cha_mrph ɲ
 	 */
-	if (Con_cost_undef == 0 || mrph_idx == new_mrph_idx) {
+	if (Cha_con_cost_undef == 0 || mrph_idx == new_mrph_idx) {
 	    if (register_undef_mrph(target + pos, new_mrph_idx) == FALSE)
 	      goto error_end;
-	    new_mrph_idx += Undef_info_num;
+	    new_mrph_idx += Cha_undef_info_num;
 	}
 #endif
 
@@ -1011,7 +873,7 @@ int chasen_sent(target, target_len, opt_nobk)
 	/* ľΥѥȤ³å */
 	for (i = mrph_idx; i < new_mrph_idx; i++) {
 	    /* ̤ƱĹñ줬ˤ̤ɲäʤ */
-	    if (Con_cost_undef > 0 && Mrph[i].length == undef_len)
+	    if (Cha_con_cost_undef > 0 && Cha_mrph[i].length == undef_len)
 	      undef_len = 0;
 	    if (check_connect(pos, i, path_idx) == FALSE)
 	      goto error_end;
@@ -1020,14 +882,14 @@ int chasen_sent(target, target_len, opt_nobk)
 	/* ̤ɲ */
 	if (undef_len > 0) {
 	    int no;
-	    for (no = 0; no < Undef_info_num; no++, new_mrph_idx++) {
+	    for (no = 0; no < Cha_undef_info_num; no++, new_mrph_idx++) {
 		if (register_undef_mrph1(target + pos, new_mrph_idx,
 					 undef_len, no) == FALSE)
 		  goto error_end;
 		if (check_connect(pos, new_mrph_idx, path_idx) == FALSE)
 		  goto error_end;
 #if 0
-		printf("path[0]: %d:%d\n",Path_num-1,Path[Path_num-1].path[0]);
+		printf("path[0]: %d:%d\n",Cha_path_num-1,Cha_path[Cha_path_num-1].path[0]);
 #endif
 	    }
 	}
@@ -1036,24 +898,24 @@ int chasen_sent(target, target_len, opt_nobk)
     }
 
     /* ʸ */
-    set_mrph_end(&Mrph[mrph_idx]);
+    set_mrph_end(&Cha_mrph[mrph_idx]);
     if (++mrph_idx % MRPH_NUM == 0 && malloc_mrph())
       goto error_end;
 
-    pos_match_process(pos_end, path_idx);
+    collect_mrphs_for_pos(pos_end, path_idx);
     if (check_connect(pos, mrph_idx - 1, path_idx) == FALSE)
       goto error_end;
 
 #ifdef DEBUG
     for (i = 1; i < mrph_idx - 1; i++) {
-	printf("%4d: %4d ", i, Mrph[i].con_tbl);
-	print_mrph(0, &Mrph[i], 'F', "%-11m %-11y %-11P3- %-14T  %F \n");
+	printf("%4d: %4d ", i, Cha_mrph[i].con_tbl);
+	print_mrph(0, &Cha_mrph[i], 'F', "%-11m %-11y %-11P3- %-14T  %F \n");
     }
 #endif
 
 #if 0
     printf("# num - mrph:%d path:%d, size - mrph:%d path:%d \n",
-	   mrph_idx, Path_num, sizeof(mrph2_t), sizeof(path_t));
+	   mrph_idx, Cha_path_num, sizeof(mrph2_t), sizeof(path_t));
 #endif
 
     return 0;
@@ -1062,7 +924,7 @@ int chasen_sent(target, target_len, opt_nobk)
     printf("Error: Too many morphs: %s\n", target);
 #if 0
     printf("# num - mrph:%d path:%d, size - mrph:%d path:%d \n",
-	   mrph_idx, Path_num, sizeof(mrph2_t), sizeof(path_t));
+	   mrph_idx, Cha_path_num, sizeof(mrph2_t), sizeof(path_t));
 #endif
     return 1;
 }
