@@ -6,6 +6,7 @@
  */
 
 #include "chalib.h"
+#include "pat.h"
 
 #define CHA_OUTPUT_SIZE (1024*16)
 
@@ -123,19 +124,21 @@ static void cha_sprintf(output, format, a, b, c, d, e, f, g, h)
     char *format, *a, *b, *c, *d, *e, *f, *g, *h;
 {
     char tmpbuf[CHA_INPUT_SIZE];
-    int len;
 
     sprintf(tmpbuf, format, a, b, c, d, e, f, g, h);
     cha_sputs(tmpbuf, output);
 }
 
-void set_cha_fput(server_mode)
+void cha_set_fput(server_mode)
     int server_mode;
 {
-#ifndef HAVE_PROTO_FPUTC
-    extern int fputc(), fputs();
+    /* 
+     * For system having no prototype declarations for the following
+     * functions such as SunOS 4.1.4. 
+     */
+    extern int fputc();
+    extern int fputs();
     extern int fprintf();
-#endif
 
     if (server_mode) {
 	cha_fputc = (void (*))cha_clputc;
@@ -148,7 +151,7 @@ void set_cha_fput(server_mode)
     }
 }
 
-void set_cha_output(output)
+void cha_set_output(output)
     FILE *output;
 {
     if (output == NULL) {
@@ -177,13 +180,13 @@ void set_cha_output(output)
 /*
  * returns cha_output for chasen_[fs]arse_tostr()
  */
-char *get_cha_output()
+char *cha_get_output()
 {
     return cha_output;
 }
 
 /*
- * printf_mrph - print morpheme using format string
+ * cha_printf_mrph - print morpheme using format string
  *
  * format string:
  *	%m     Ф(ѷ)
@@ -197,8 +200,8 @@ char *get_cha_output()
  *	%a0    ȯ(ѷ)
  *	%A0    ȯ(ܷ)
  *      %rabc  ӤĤθФ("aФbɤc" ɽ)
- *	%i     ̣
- *	%Ic    ̣(ʸ"NIL"ʤʸc)
+ *	%i     ղþ
+ *	%Ic    ղþ(ʸ"NIL"ʤʸc)
  *	%Pc    ƳؤʻʸcǶڤäʸ(vgramΤ)
  *	%Pnc   1n(n:19)ܤޤǤʻʸcǶڤäʸ(vgramΤ)
  *	%h     ʻֹ
@@ -224,7 +227,7 @@ char *get_cha_output()
  *	       ɽʤ
  *
  *	%?B/STR1/STR2/  ʻʬबSTR1ʤSTR2
- *	%?I/STR1/STR2/  ̣NILǤ""ǤʤSTR1ǤʤSTR2
+ *	%?I/STR1/STR2/  ղþNILǤ""ǤʤSTR1ǤʤSTR2
  *	%?T/STR1/STR2/  ѤSTR1ʤSTR2
  *	%?F/STR1/STR2/  ѤSTR1ʤSTR2
  *	%?U/STR1/STR2/  ̤ʤSTR1ǤʤSTR2
@@ -254,13 +257,13 @@ static int check_con_cost(path, con_tbl)
 {
     int con_cost;
 
-    check_automaton(path->state,
+    cha_check_automaton(path->state,
 #ifdef KOCHA
-		    Path[path->path[0]].state,
-		    Mrph[path->mrph_p].midasi,
-		    Mrph[path->mrph_p].is_undef,
+		    Cha_path[path->path[0]].state,
+		    Cha_mrph[path->mrph_p].midasi,
+		    Cha_mrph[path->mrph_p].is_undef,
 #endif
-		    con_tbl, Con_cost_undef, &con_cost);
+		    con_tbl, Cha_con_cost_undef, &con_cost);
 
     return con_cost;
 }
@@ -333,7 +336,6 @@ static void set_ruby(dest, midasi, yomi, par1, par2, par3)
     *d = '\0';
 }
 
-#ifdef VGRAM
 static void print_nhinsi(hinsi, c, n)
     int hinsi, c, n;
 {
@@ -343,9 +345,9 @@ static void print_nhinsi(hinsi, c, n)
     if (c == '\'')
       cha_putc(c, cha_output);
 
-    path = Hinsi[hinsi].path;
+    path = Cha_hinsi[hinsi].path;
     for (i = 0; ; i++) {
-	cha_puts(Hinsi[*path].name, cha_output);
+	cha_puts(Cha_hinsi[*path].name, cha_output);
 	if (!*path || !*++path || i == n)
 	  break;
 	if (c == '\'')
@@ -357,8 +359,6 @@ static void print_nhinsi(hinsi, c, n)
     if (c == '\'')
       cha_putc(c, cha_output);
 }
-#endif
-
 
 /***********************************************************************
  * int_to_str - convert an integer to ASCII
@@ -496,11 +496,11 @@ static void print_anno(path_num, format)
     int path_num;
     char *format;
 {
-    path_t *path = &Path[path_num];
+    path_t *path = &Cha_path[path_num];
     mrph2_t mrph;
     int start, end;
 
-    if (!Anno_info[0].hinsi && !Anno_info[1].hinsi)
+    if (!Cha_anno_info[0].hinsi && !Cha_anno_info[1].hinsi && !Cha_anno_info[1].format)
       return;
 
     if (path->start <= pos_end) {
@@ -513,27 +513,30 @@ static void print_anno(path_num, format)
 
     while (start > pos_end) {
 	int anno_no = -char_type[pos_end];
-	if (Anno_info[anno_no].hinsi) {
+	char *format_string = format;
+	if (anno_no >=0 &&
+	    (Cha_anno_info[anno_no].hinsi || Cha_anno_info[anno_no].format)) {
 	    mrph.midasi = sentence + pos_end;
-	    mrph.yomi = "";
 	    mrph.base_length = mrph.length = undefword_len[pos_end];
-#ifdef VGRAM
+	    mrph.yomi = "";
 	    mrph.base = "";
 	    mrph.pron = "";
-#endif
-	    mrph.hinsi = Anno_info[anno_no].hinsi;
-#ifndef VGRAM
-	    mrph.bunrui = Anno_info[anno_no].bunrui;
-#endif
+	    if (Cha_anno_info[anno_no].format) {
+		format_string = Cha_anno_info[anno_no].format;
+		mrph.is_undef = 1;
+		mrph.hinsi = Cha_undef_info[0].hinsi;
+	    } else {
+		mrph.is_undef = 0;
+		mrph.hinsi = Cha_anno_info[anno_no].hinsi;
+	    }
 	    mrph.con_tbl = 0;
 	    mrph.ktype = 0;
 	    mrph.kform = 0;
-	    mrph.is_undef = 0;
 	    mrph.weight = 0;
 	    mrph.info = "";
 	    path->start = pos_end;
 	    path->end = pos_end + mrph.length;
-	    printf_mrph(path_num, &mrph, format);
+	    cha_printf_mrph(path_num, &mrph, format_string);
 	}
 	pos_end += undefword_len[pos_end];
     }
@@ -572,26 +575,17 @@ static void extract_yomi1(dst, src)
     *d = '\0';
 }
 
-static void print_mrph(path_num, mrph, format)
+void cha_printf_mrph(path_num, mrph, format)
     int path_num;
     mrph2_t *mrph;
     char *format;
 {
-    print_anno(path_num, format);
-    printf_mrph(path_num, mrph, format);
-}
-
-void printf_mrph(path_num, mrph, format)
-    int path_num;
-    mrph2_t *mrph;
-    char *format;
-{
-    int  letter, value, n, c, state;
+    int  letter, value, n, state;
     int  deli_left, deli_right;
-    char *s, *sp, *word, *eword;
+    char *s, *word, *eword;
     char word_str[CHA_INPUT_SIZE], word_str2[CHA_INPUT_SIZE];
     char *width_str;
-    path_t *path = &Path[path_num];
+    path_t *path = &Cha_path[path_num];
 
     eword = NULL;     /* string in EUC */
     word = NULL;      /* string in EUC(UNIX) or SJIS(Win) */
@@ -608,7 +602,6 @@ void printf_mrph(path_num, mrph, format)
 	    continue;
 	}
 #endif
-
 	if (state == 1 && *s == deli_right) {
 	    if (deli_right != deli_left && !*s++)
 	      return;
@@ -647,13 +640,8 @@ void printf_mrph(path_num, mrph, format)
 		  state = 1;
 		break;
 	      case 'B':
-#ifdef VGRAM
-		if (Hinsi[mrph->hinsi].depth > 1)
+		if (Cha_hinsi[mrph->hinsi].depth > 1)
 		  state = 1;
-#else
-		if (mrph->bunrui)
-		  state = 1;
-#endif
 		break;
 	      case 'I':
 		if (mrph->info[0] && strcmp(mrph->info, "NIL"))
@@ -683,7 +671,7 @@ void printf_mrph(path_num, mrph, format)
 		deli_right = *s;
 		deli_left = '\0';
 		s--;
-	        word = Lang_e ? ESTR_UNKNOWN_WORD : JSTR_UNKNOWN_WORD;
+	        word = Cha_lang_e ? ESTR_UNKNOWN_WORD : JSTR_UNKNOWN_WORD;
 	    } else {
 	        state = 2;
 		deli_right = get_deli_right(deli_left = *++s);
@@ -702,17 +690,15 @@ void printf_mrph(path_num, mrph, format)
 	    /* bunsetsu */
 	    if (mrph->length == 0)
 	      word = mrph->midasi;
-#ifdef VGRAM
 	    else if (mrph->base[0])
 	      eword = mrph->base;
-#endif
 	    else {
 		memcpy(eword = word_str, mrph->midasi, mrph->base_length);
 		if (!mrph->ktype)
 		  word_str[mrph->base_length] = '\0';
 		else
 		  strcpy(word_str + mrph->base_length,
-			 Form[mrph->ktype][Type[mrph->ktype].basic].gobi);
+			 Cha_form[mrph->ktype][Cha_type[mrph->ktype].basic].gobi);
 	    }
 	    break;
 	  case 'y': /* ɤ */
@@ -723,28 +709,20 @@ void printf_mrph(path_num, mrph, format)
 	      word = mrph->midasi;
 	    else {
 		if (mrph->yomi[0]) {
-#ifdef VGRAM
 		    if (s[0] != 'r' && s[1] != '0')
 		      extract_yomi1(word_str, mrph->yomi);
 		    else
 		      strcpy(word_str, mrph->yomi);
-#else
-		    sp = strchr(mrph->yomi, ' ');
-		    memcpy(word_str, mrph->yomi, sp - mrph->yomi);
-		    word_str[sp - mrph->yomi] = '\0';
-		    if (s[0] != 'r' && s[1] != '0')
-		      extract_yomi1(word_str, word_str);
-#endif
 		} else {
 		    memcpy(word_str, mrph->midasi, mrph->base_length);
 		    word_str[mrph->base_length] = '\0';
 		}
 		if (mrph->ktype > 0) {
 		    if (*s != 'Y')
-		      strcat(word_str, Form[mrph->ktype][mrph->kform].ygobi);
+		      strcat(word_str, Cha_form[mrph->ktype][mrph->kform].ygobi);
 		    else
 		      strcat(word_str,
-			     Form[mrph->ktype][Type[mrph->ktype].basic].ygobi);
+			     Cha_form[mrph->ktype][Cha_type[mrph->ktype].basic].ygobi);
 		}
 		eword = word_str;
 	    }
@@ -771,7 +749,6 @@ void printf_mrph(path_num, mrph, format)
 	    }
 	    break;
 #endif
-#ifdef VGRAM
 	  case 'a': /* ȯ */
 	  case 'A': /* ȯ(ܷ) */
 	    /* bunsetsu */
@@ -794,21 +771,20 @@ void printf_mrph(path_num, mrph, format)
 		}
 		if (mrph->ktype > 0) {
 		    if (*s != 'A')
-		      strcat(word_str, Form[mrph->ktype][mrph->kform].pgobi);
+		      strcat(word_str, Cha_form[mrph->ktype][mrph->kform].pgobi);
 		    else
 		      strcat(word_str,
-			     Form[mrph->ktype][Type[mrph->ktype].basic].pgobi);
+			     Cha_form[mrph->ktype][Cha_type[mrph->ktype].basic].pgobi);
 		}
 		eword = word_str;
 	    }
 	    if (s[1] == '0' || s[1] == '1')
 	      s++;
 	    break;
-#endif
-	  case 'i': /* ̣ */
+	  case 'i': /* ղþ */
 	    eword = mrph->info;
 	    break;
-	  case 'I': /* ̣ */
+	  case 'I': /* ղþ */
 	    if (*++s == '\0')
 	      cha_putc(*--s, cha_output);
 	    else if (mrph->info[0] && strcmp(mrph->info, "NIL"))
@@ -816,7 +792,6 @@ void printf_mrph(path_num, mrph, format)
 	    else if (*s != ' ')
 	      letter = *s;
 	    break;
-#ifdef VGRAM
 	  case 'P':
 	    n = 99; /* ٤Ƥγؤɽ */
 	    if (s[1] >= '1' && s[1] <= '9')
@@ -826,44 +801,30 @@ void printf_mrph(path_num, mrph, format)
 	    else
 	      print_nhinsi(mrph->hinsi, *++s, n);
 	    break;
-#endif
 	  case 'h': /* ʻ(ֹ) */
 	    value = mrph->hinsi;
 	    break;
 	  case 'H': /* ʻ(ʸ) */
-#ifdef VGRAM
 	    if (s[1] < '1' || s[1] > '9')
 	      n = 0;
 	    else {
 		n = *++s - '1';
-		if (Hinsi[mrph->hinsi].depth - 1 < n)
-		  n = Hinsi[mrph->hinsi].depth - 1;
+		if (Cha_hinsi[mrph->hinsi].depth - 1 < n)
+		  n = Cha_hinsi[mrph->hinsi].depth - 1;
 	    }
-	    word = Hinsi[Hinsi[mrph->hinsi].path[n]].name;
-#else
-	    word = Class[mrph->hinsi][0].id;
-#endif
+	    word = Cha_hinsi[Cha_hinsi[mrph->hinsi].path[n]].name;
 	    break;
 	  case 'b': /* ʻʬ(ֹ) */
-#ifdef VGRAM
 	    value = 0;
-#else
-	    value = mrph->bunrui;
-#endif
 	    break;
 	  case 'B': /* ʻʬ(ʸ) */
 	    if (s[1] == '\0')
 	      cha_putc(*s, cha_output);
 	    else if (*++s == 'M' && mrph->is_undef)
-	      word = Lang_e ? ESTR_UNKNOWN_WORD : JSTR_UNKNOWN_WORD;
-#ifdef VGRAM
+	      word = Cha_lang_e ? ESTR_UNKNOWN_WORD : JSTR_UNKNOWN_WORD;
 	    /* زʻʤֲγؤʻ̾ɽ */
-	    else if (*s == 'M' || *s == 'B' || Hinsi[mrph->hinsi].depth > 1)
-	      word = Hinsi[mrph->hinsi].name;
-#else
-	    else if (*s == 'M' || *s == 'B' || mrph->bunrui)
-	      word = Class[mrph->hinsi][mrph->bunrui].id;
-#endif
+	    else if (*s == 'M' || *s == 'B' || Cha_hinsi[mrph->hinsi].depth > 1)
+	      word = Cha_hinsi[mrph->hinsi].name;
 	    else if (*s != ' ')
 	      letter = *s;
 	    break;
@@ -874,7 +835,7 @@ void printf_mrph(path_num, mrph, format)
 	    if (*++s == '\0')
 	      cha_putc(*--s, cha_output);
 	    else if (mrph->ktype)
-	      word = Type[mrph->ktype].name;
+	      word = Cha_type[mrph->ktype].name;
 	    else if (*s != ' ')
 	      letter = *s;
 	    break;
@@ -885,22 +846,18 @@ void printf_mrph(path_num, mrph, format)
 	    if (*++s == '\0')
 	      cha_putc(*--s, cha_output);
 	    if (mrph->kform)
-	      word = Form[mrph->ktype][mrph->kform].name;
+	      word = Cha_form[mrph->ktype][mrph->kform].name;
 	    else if (*s != ' ')
 	      letter = *s;
 	    break;
 	  case 'c': /* ǤΥ */
 	    if (mrph->is_undef) {
-		value = Undef_info[mrph->is_undef-1].cost
-		  + Undef_info[mrph->is_undef-1].cost_step * mrph->length / 2;
+		value = Cha_undef_info[mrph->is_undef-1].cost
+		  + Cha_undef_info[mrph->is_undef-1].cost_step * mrph->length / 2;
 	    } else {
-#ifdef VGRAM
-		value = Hinsi[mrph->hinsi].cost;
-#else
-		value = Class[mrph->hinsi][mrph->bunrui].cost;
-#endif
+		value = Cha_hinsi[mrph->hinsi].cost;
 	    }
-	    value *= mrph->weight * Mrph_cost_weight;
+	    value *= mrph->weight * Cha_mrph_cost_weight;
 	    break;
 	  case 'S': /* ʸ */
 	    word = sentence;
@@ -942,8 +899,8 @@ void printf_mrph(path_num, mrph, format)
 		    for (j = 0; path->path[j] != -1; j++) {
 			if (j)
 			  cha_putc(c, cha_output);
-			cha_printf(cha_output, "%d", Con_cost_weight *
-				   check_con_cost(&Path[path->path[j]], con_tbl));
+			cha_printf(cha_output, "%d", Cha_con_cost_weight *
+				   check_con_cost(&Cha_path[path->path[j]], con_tbl));
 		    }
 		}		
 		break;
@@ -1008,8 +965,8 @@ static void print_bos_eos(str)
 static void print_bos(opt_form)
     int opt_form;
 {
-    if (opt_form != 'W' && opt_form != 'd' && *Bos_string)
-      print_bos_eos(Bos_string);
+    if (opt_form != 'W' && opt_form != 'd' && *Cha_bos_string)
+      print_bos_eos(Cha_bos_string);
 }
 
 static void print_eos(opt_form)
@@ -1017,30 +974,52 @@ static void print_eos(opt_form)
 {
     if (opt_form == 'W')
       cha_putc('\n', cha_output);
-    else if (opt_form != 'd' && *Eos_string)
-      print_bos_eos(Eos_string);
+    else if (opt_form != 'd' && *Cha_eos_string)
+      print_bos_eos(Cha_eos_string);
 }
 
 /*
-------------------------------------------------------------------------------
-  PROCEDURE: <print_path_mrph>
-------------------------------------------------------------------------------
-*/
+ * print_path_mrph
+ */
+static void print_mrph(path_num, mrph, format)
+    int path_num;
+    mrph2_t *mrph;
+    char *format;
+{
+    print_anno(path_num, format);
+    if (Cha_output_compo || *mrph->comp == '\n')
+      cha_printf_mrph(path_num, mrph, format);
+    else {
+	/* compound word */
+	int kform = mrph->kform;
+	while (*mrph->comp != '\n') {
+	    cha_get_mrph_data(mrph, mrph->comp, mrph->midasi);
+	    /* ǸηǤγѷʣγѷ */
+	    if (*mrph->comp == '\n' && !mrph->kform)
+	      mrph->kform = kform;
+	    if (mrph->ktype) {
+		mrph->length += strlen(Cha_form[mrph->ktype][mrph->kform].gobi);
+		mrph->con_tbl += mrph->kform - 1;
+	    }
+	    cha_printf_mrph(path_num, mrph, format);
+	    mrph->midasi += mrph->length;
+	}
+    }
+}
+
 static void print_path_mrph(path_num, format)
     int path_num;
     char *format;
 {
 #ifdef KOCHA_DEBUG
-    printf("%4d: ", Path[path_num].mrph_p);
+    printf("%4d: ", Cha_path[path_num].mrph_p);
 #endif
-    print_mrph(path_num, &Mrph[Path[path_num].mrph_p], format);
+    print_mrph(path_num, &Cha_mrph[Cha_path[path_num].mrph_p], format);
 }
 
 /*
-------------------------------------------------------------------------------
-  PROCEDURE: <print_best_path> ǾȤΥѥΤߤɽ
-------------------------------------------------------------------------------
-*/
+ * print_best_path()
+ */
 static void print_best_path(opt_form, format)
     int opt_form;
     char *format;
@@ -1054,51 +1033,32 @@ static void print_best_path(opt_form, format)
 
     print_bos(opt_form);
 
-    last = Path[Path_num - 1].path[0];
+    last = Cha_path[Cha_path_num - 1].path[0];
     if (last) {
-	for (pbuf_last = 0; last; last = Path[last].path[0], pbuf_last++) {
+	for (pbuf_last = 0; last; last = Cha_path[last].path[0], pbuf_last++) {
 	    path_buffer[pbuf_last] = last;
 #if 0
-	    printf("# last,path0: %d,%d\n",last,Path[last].path[0]);fflush(stdout);
+	    printf("# last,path0: %d,%d\n",last,Cha_path[last].path[0]);fflush(stdout);
 #endif
 	}
 
 	/* ϢʻñϢ뤷ɽ */
 	mrph.hinsi = 0;
 	mrph.yomi = yomi;
-#ifdef VGRAM
 	mrph.base = base;
 	mrph.pron = pron;
-#endif
-	mrph1 = &Mrph[Path[path_buffer[pbuf_last - 1]].mrph_p];
+	mrph1 = &Cha_mrph[Cha_path[path_buffer[pbuf_last - 1]].mrph_p];
 	for (i = pbuf_last - 1; i >= 0; i--) {
-	    mrph2 = i == 0 ? NULL : &Mrph[Path[path_buffer[i - 1]].mrph_p];
+	    mrph2 = i == 0 ? NULL : &Cha_mrph[Cha_path[path_buffer[i - 1]].mrph_p];
 	    if (i > 0 &&
 		!mrph1->is_undef && !mrph2->is_undef &&
-		mrph1->midasi[mrph1->length] != ' ' &&
-		mrph1->midasi[mrph1->length] != '\t' &&
-#ifdef VGRAM
-		Hinsi[mrph1->hinsi].comp &&
-		Hinsi[mrph1->hinsi].comp == Hinsi[mrph2->hinsi].comp
-#else
-		Class[mrph1->hinsi][mrph1->bunrui].comp &&
-		mrph1->hinsi == mrph2->hinsi &&
-		(Class[mrph1->hinsi][mrph1->bunrui].comp == 1 ||
-		 mrph1->bunrui == mrph2->bunrui)
-#endif
-		) {
+		Cha_path[path_buffer[i]].end == Cha_path[path_buffer[i - 1]].start &&
+		Cha_hinsi[mrph1->hinsi].comp &&
+		Cha_hinsi[mrph1->hinsi].comp == Cha_hinsi[mrph2->hinsi].comp) {
 		if (!mrph.hinsi) {
-#ifdef VGRAM
-		    mrph.hinsi = Hinsi[mrph1->hinsi].comp;
+		    mrph.hinsi = Cha_hinsi[mrph1->hinsi].comp;
 		    base[0] = '\0';
 		    pron[0] = '\0';
-#else
-		    mrph.hinsi = mrph1->hinsi;
-		    if (Class[mrph1->hinsi][mrph1->bunrui].comp == 1)
-		      mrph.bunrui = 0;
-		    else
-		      mrph.bunrui = mrph1->bunrui;
-#endif
 		    yomi[0] = '\0';
 		    mrph.midasi = mrph1->midasi;
 		    mrph.length = mrph.weight = 0;
@@ -1112,8 +1072,7 @@ static void print_best_path(opt_form, format)
 		    yomi[len + mrph1->base_length] = '\0';
 		}
 		if (mrph1->ktype > 0)
-		  strcat(yomi, Form[mrph1->ktype][mrph1->kform].ygobi);
-#ifdef VGRAM
+		  strcat(yomi, Cha_form[mrph1->ktype][mrph1->kform].ygobi);
 		strcat(base, mrph1->base);
 		if (mrph1->pron[0])
 		  strcat(pron, mrph1->pron);
@@ -1125,8 +1084,7 @@ static void print_best_path(opt_form, format)
 		    pron[len + mrph1->base_length] = '\0';
 		}
 		if (mrph1->ktype > 0)
-		  strcat(pron, Form[mrph1->ktype][mrph1->kform].pgobi);
-#endif
+		  strcat(pron, Cha_form[mrph1->ktype][mrph1->kform].pgobi);
 		mrph.length += mrph1->length;
 		mrph.weight += mrph1->weight;
 	    } else {
@@ -1146,7 +1104,6 @@ static void print_best_path(opt_form, format)
 			memcpy(yomi + len, mrph1->midasi, mrph1->base_length);
 			yomi[len + mrph1->base_length] = '\0';
 		    }
-#ifdef VGRAM
 		    strcat(base, mrph1->base);
 		    if (mrph1->pron[0])
 		      strcat(pron, mrph1->pron);
@@ -1157,7 +1114,6 @@ static void print_best_path(opt_form, format)
 			memcpy(pron + len, mrph1->midasi, mrph1->base_length);
 			pron[len + mrph1->base_length] = '\0';
 		    }
-#endif
 		    mrph.base_length = mrph.length + mrph1->base_length;
 		    mrph.length += mrph1->length;
 		    mrph.weight += mrph1->weight;
@@ -1165,8 +1121,8 @@ static void print_best_path(opt_form, format)
 		    mrph.ktype = mrph1->ktype;
 		    mrph.kform = mrph1->kform;
 		    mrph.is_undef = mrph1->is_undef;
-		    Path[path_num_comp].end = Path[path_num_comp].start + mrph.length;
-		    print_mrph(path_buffer[i], &mrph, format);
+		    Cha_path[path_num_comp].end = Cha_path[path_num_comp].start + mrph.length;
+		    print_mrph(path_num_comp, &mrph, format);
 		    mrph.hinsi = 0;
 		}
 	    }
@@ -1174,7 +1130,7 @@ static void print_best_path(opt_form, format)
 	}
     }
 
-    print_anno(Path_num - 1, format);
+    print_anno(Cha_path_num - 1, format);
     print_eos(opt_form);
 }
 
@@ -1187,11 +1143,11 @@ static void collect_all_mrph(path_num)
 {
     int i, j;
 
-    for (i = 0; (j = Path[path_num].path[i]) && j != -1; i++) {
-	if (!Path[j].do_print) {
-	    Path[j].do_print =
+    for (i = 0; (j = Cha_path[path_num].path[i]) && j != -1; i++) {
+	if (!Cha_path[j].do_print) {
+	    Cha_path[j].do_print =
 	      (i == 0 &&
-	       (path_num == Path_num - 1 || Path[path_num].do_print == 2))
+	       (path_num == Cha_path_num - 1 || Cha_path[path_num].do_print == 2))
 		? 2 : 1;
 	    collect_all_mrph(j);
 	}
@@ -1202,22 +1158,22 @@ static void print_all_mrph(opt_form, format)
     int opt_form;
     char *format;
 {
-    int i, j;
+    int i;
     int isfirst = 1; /* ʸƬɤΥե饰 for -d option */
 
-    for (i = 0; i < Path_num; i++)
-      Path[i].do_print = 0;
-    collect_all_mrph(Path_num - 1);
+    for (i = 0; i < Cha_path_num; i++)
+      Cha_path[i].do_print = 0;
+    collect_all_mrph(Cha_path_num - 1);
 
     /* -v ΤȤʸƬʸξɽ */
     if (opt_form == 'v') {
-	Path[0].do_print = 2;
-	Path[Path_num - 1].do_print = 2;
+	Cha_path[0].do_print = 2;
+	Cha_path[Cha_path_num - 1].do_print = 2;
     }
 
     print_bos(opt_form);
-    for (i = 0; i < Path_num; i++) {
-	if (Path[i].do_print) {
+    for (i = 0; i < Cha_path_num; i++) {
+	if (Cha_path[i].do_print) {
 	    if (opt_form == 'd') {
 		if (isfirst)
 		  isfirst = 0;
@@ -1227,31 +1183,29 @@ static void print_all_mrph(opt_form, format)
 	    print_path_mrph(i, format);
 	}
     }
-    print_anno(Path_num - 1, format);
+    print_anno(Cha_path_num - 1, format);
     print_eos(opt_form);
 }
 
 /*
-------------------------------------------------------------------------------
-  PROCEDURE: <print_all_path> Ϸ̤˴ޤޤƤPATH
-------------------------------------------------------------------------------
-*/
+ * print_all_path()
+ */
 static void print_all_path_sub(path_num, paths, opt_form, format)
     int path_num, paths, opt_form;
     char *format;
 {
     int i, j;
 
-    for (i = 0; Path[path_num].path[i] != -1; i++) {
-	if (Path[path_num].path[0] == 0) {
+    for (i = 0; Cha_path[path_num].path[i] != -1; i++) {
+	if (Cha_path[path_num].path[0] == 0) {
 	    pos_end = 0;
 	    for (j = paths - 1; j >= 0; j--)
 	      print_path_mrph(path_buffer[j], format);
-	    print_anno(Path_num - 1, format);
+	    print_anno(Cha_path_num - 1, format);
 	    cha_puts("EOP\n", cha_output);
 	} else {
-	    path_buffer[paths] = Path[path_num].path[i];
-	    print_all_path_sub(Path[path_num].path[i], paths + 1,
+	    path_buffer[paths] = Cha_path[path_num].path[i];
+	    print_all_path_sub(Cha_path[path_num].path[i], paths + 1,
 			       opt_form, format);
 	}
     }
@@ -1262,7 +1216,7 @@ static void print_all_path(opt_form, format)
     char *format;
 {
     print_bos(opt_form);
-    print_all_path_sub(Path_num - 1, 0, opt_form, format);
+    print_all_path_sub(Cha_path_num - 1, 0, opt_form, format);
     print_eos(opt_form, format);
 }
 
@@ -1293,34 +1247,27 @@ void cha_print_bos_eos(opt_form)
 
 void cha_print_hinsi_table()
 {
-    int i, j;
-#ifdef VGRAM
-      for (i = 0; Hinsi[i].name; i++) {
-	  cha_printf(cha_output, "%d ", i);
-	  print_nhinsi(i, '-', 99);
-	  cha_putc('\n', cha_output);
-      }
-#else
-    for (i = 0; Class[i][0].id; i++) {
-	printf("%d 0 %s\n", i, Class[i][0].id);
-	for (j = 1; Class[i][j].id; j++)
-	  printf("%d %d %s-%s\n", i, j, Class[i][0].id, Class[i][j].id);
+    int i;
+
+    for (i = 0; Cha_hinsi[i].name; i++) {
+	cha_printf(cha_output, "%d ", i);
+	print_nhinsi(i, '-', 99);
+	cha_putc('\n', cha_output);
     }
-#endif
 }
 
 void cha_print_ctype_table()
 {
     int i;
-    for (i = 1; Type[i].name; i++)
-      cha_printf(cha_output, "%d %s\n", i, Type[i].name);
+    for (i = 1; Cha_type[i].name; i++)
+      cha_printf(cha_output, "%d %s\n", i, Cha_type[i].name);
 }
 
 void cha_print_cform_table()
 {
     int i, j;
-    for (i = 1; Type[i].name; i++)
-      for (j = 1; Form[i][j].name; j++)
-	printf("%d %d %s\n", i, j, Form[i][j].name);
+    for (i = 1; Cha_type[i].name; i++)
+      for (j = 1; Cha_form[i][j].name; j++)
+	printf("%d %d %s\n", i, j, Cha_form[i][j].name);
 }
 
